﻿/*
	This file is part of Warzone Stat Editor by crab
    Copyright © 2011, 2012 Vladimir Salnikov

	Warzone Stat Editor by crab is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	Warzone Stat Editor by crab is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Warzone Stat Editor by crab; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

namespace WZStats
{
    public class ClassResearch
    {
        Stats stats;
        //public static string[] ItemFiles = { "weapons.txt"};

        public ClassResearch(Stats s)
        {
            stats = s;
        }



        int recursion_counter;
        List<DataRow> ResearchPrerequisites(DataRow ResearchRow)
        {
            if (recursion_counter > 2000)
                return null;
            recursion_counter++;
            DataTable dt_res = stats.FileList["research.txt"].data;
            DataTable dt_pre = stats.FileList["prresearch.txt"].data;
            DataRow[] pre_rows = dt_pre.Select(String.Format("ResearchCode = '{0}'", ResearchRow["Code"].ToString()));
            if (pre_rows.Length == 0)
                return null;
            List<DataRow> result = new List<DataRow>();
            foreach (DataRow row in pre_rows)
            {
                DataRow[] res_rows = dt_res.Select(String.Format("Code = '{0}'", row["PreresearchCode"].ToString()));
                if(res_rows.Length > 0)
                {
                    DataRow res_row = res_rows[0];
                    result.Add(res_row);
                    List<DataRow> new_items = ResearchPrerequisites(res_row);
                    if(new_items != null)
                        result.AddRange
                            (new_items);
                }
            }
            
            return result;
        }

        DataRow ResearchRow(string itemCode)
        {
            if (!stats.FileList["research.txt"].Loaded)
                return null;
            DataTable dt_res = stats.FileList["research.txt"].data;

            DataTable dt;
            if (stats.FileList["resultstructure.txt"].Loaded)
            {
                dt = stats.FileList["resultstructure.txt"].data;
                foreach (DataRow row in dt.Rows)
                {
                    if (row["StructureCode"].ToString() == itemCode)
                    {
                        DataRow[] res_rows = dt_res.Select(String.Format("Code = '{0}'", row["ResearchCode"].ToString()));
                        if(res_rows.Length > 0)
                            return res_rows[0];
                    }
                }
            }

            if (stats.FileList["resultcomponent.txt"].Loaded)
            {
                dt = stats.FileList["resultcomponent.txt"].data;
                foreach (DataRow row in dt.Rows)
                {
                    if (row["ComponentCode"].ToString() == itemCode)
                    {
                        DataRow[] res_rows = dt_res.Select(String.Format("Code = '{0}'", row["ResearchCode"].ToString()));
                        if (res_rows.Length > 0)
                            return res_rows[0];
                    }
                }
            }

            //may be item code is code of research!
            dt = stats.FileList["research.txt"].data;
            foreach (DataRow row in dt.Rows)
                if (row["Code"].ToString() == itemCode)
                    return row;


            return null;
        }

        public int ComputeResearchPoints(string ItemCode)
        {
            if (!stats.FileList["research.txt"].Loaded || !stats.FileList["prresearch.txt"].Loaded)
                return -1;
            DataRow res_row = ResearchRow(ItemCode);
            if (res_row == null)
                return -1;
            recursion_counter = 0;
            List<DataRow> pre_res = ResearchPrerequisites(res_row);
            Dictionary<string, DataRow> pre_res_unique = new Dictionary<string, DataRow>();
            if (pre_res != null)
            {
                foreach (DataRow row in pre_res)
                {
                    if (!pre_res_unique.ContainsKey(row["Code"].ToString()))
                        pre_res_unique.Add(row["Code"].ToString(), row);
                }
            }
            
            int res_points = Convert.ToInt32(res_row["Research points"]);
            foreach(DataRow row in pre_res_unique.Values)
                res_points += Convert.ToInt32(row["Research points"]);

            return res_points;
        }


        public DataTable MakeResTreeForItem(string ItemCode)
        {

            if (!stats.FileList["research.txt"].Loaded || !stats.FileList["prresearch.txt"].Loaded)
                return null;
            DataTable dt_res = stats.FileList["research.txt"].data;
            DataTable ResTree = dt_res.Copy();
            ResTree.TableName = "Research Tree for one item";
            ResTree.Rows.Clear();
            ResTree.Columns.Add("ID", typeof(Int32));
            ResTree.Columns.Add("ParentID", typeof(Int32));
            for(int i=0; i<ResTree.Columns.Count; i++)
                ResTree.Columns[i].Unique = false; //column "Code" and other columns can be unique in research.txt
            DataRow res_row = ResearchRow(ItemCode);
            if (res_row == null)
                return ResTree;

            recursion_counter = 0;
            ResTree.ImportRow(res_row);
            int ID = 0;
            ResTree.Rows[0]["ID"] = ID;
            List<DataRow> pre_res = ResearchPrerequisites_asTree(res_row, ResTree, ref ID);
            if(pre_res != null)
                foreach(DataRow row in pre_res)
                {
                    ResTree.Rows.Add(row);
                }

            return ResTree;
        }

        List<DataRow> ResearchPrerequisites_asTree(DataRow ResearchRow, DataTable ResTree, ref int ID)
        {
            if (recursion_counter > 2000)
                return null;
            recursion_counter++;
            int parent_id = ID;
            ID = ID + 1;
            DataTable dt_res = stats.FileList["research.txt"].data;
            DataTable dt_pre = stats.FileList["prresearch.txt"].data;
            DataRow[] pre_rows = dt_pre.Select(String.Format("ResearchCode = '{0}'", (string)ResearchRow["Code"]));
            if (pre_rows.Length == 0)
                return null;
            List<DataRow> result = new List<DataRow>();
            foreach (DataRow row in pre_rows)
            {
                DataRow[] res_rows = dt_res.Select(String.Format("Code = '{0}'", row["PreresearchCode"].ToString()));
                if(res_rows.Length > 0)
                {
                    DataRow res_row = res_rows[0];
                    DataRow tree_row = ResTree.NewRow();
                    for(int i = 0; i < res_row.Table.Columns.Count; i++)
                        tree_row[i] = res_row[i];
                    tree_row["ParentID"] = parent_id;
                    tree_row["ID"] = ID;
                    result.Add(tree_row);
                    List<DataRow> new_items = ResearchPrerequisites_asTree(res_row, ResTree, ref ID);
                    if (new_items != null)
                        result.AddRange
                            (new_items);
                }
            }

            return result;
        }

    }

}
