;
;	NSIS Modern User Interface
;	Installer script
;

;--------------------------------
;Include section
!include "MUI2.nsh"

;--------------------------------
;Defines
!define PRODUCT_NAME "Warzone Stat Editor"
!define PRODUCT_VERSION "5.1" ;MUST be X.X
!define PRODUCT_DIR_REGKEY "Software\${PRODUCT_NAME}\${PRODUCT_VERSION}"
!define PRODUCT_UNINST_KEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PRODUCT_NAME}"
!define PRODUCT_UNINST_ROOT_KEY "HKLM"



;--------------------------------
;General
CRCCheck on	 ;make sure this isn't corrupted
SetCompressor /SOLID	lzma

;Name and file
Name "${PRODUCT_NAME} ${PRODUCT_VERSION}"
OutFile "Install ${PRODUCT_NAME} ${PRODUCT_VERSION}.exe"

;Default installation folder
InstallDir "$PROGRAMFILES\${PRODUCT_NAME}"

;Get installation folder from registry if available
InstallDirRegKey HKLM "${PRODUCT_DIR_REGKEY}" ""

;Request application privileges for Windows Vista
RequestExecutionLevel admin

Name "${PRODUCT_NAME} ${PRODUCT_VERSION}"
OutFile "Install ${PRODUCT_NAME} v${PRODUCT_VERSION}.exe"


;--------------------------------
;Versioninfo
VIProductVersion "${PRODUCT_VERSION}.0.0"
VIAddVersionKey "CompanyName"		""
VIAddVersionKey "FileDescription"	"${PRODUCT_NAME} v${PRODUCT_VERSION} Installer"
VIAddVersionKey "FileVersion"		"${PRODUCT_VERSION}"
VIAddVersionKey "InternalName"		"${PRODUCT_NAME} v${PRODUCT_VERSION}"
VIAddVersionKey "LegalCopyright"	"${PRODUCT_NAME} Copyright  2011-2012 "
VIAddVersionKey "OriginalFilename"	"${PRODUCT_NAME} ${PRODUCT_VERSION}.exe"
VIAddVersionKey "ProductName"		"${PRODUCT_NAME} v${PRODUCT_VERSION}"
VIAddVersionKey "ProductVersion"	"${PRODUCT_VERSION}"
VIAddVersionKey "Installer"			"Installer script Copyright  2012 Milo Christiansen"

;--------------------------------
;Variables
Var StartMenuFolder
	
;--------------------------------
;Interface Settings
!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP "Resources\header.bmp"
!define MUI_HEADERIMAGE_UNBITMAP "Resources\header.un.bmp"
!define MUI_HEADERIMAGE_RIGHT

!define MUI_WELCOMEFINISHPAGE_BITMAP "Resources\welcome.bmp"
!define MUI_UNWELCOMEFINISHPAGE_BITMAP "Resources\welcome.un.bmp"

!define MUI_ICON "Resources\install.ico"
!define MUI_UNICON "Resources\uninstall.ico"

!define MUI_ABORTWARNING

; These indented statements modify settings for MUI_PAGE_FINISH
!define MUI_FINISHPAGE_NOAUTOCLOSE
!define MUI_UNFINISHPAGE_NOAUTOCLOSE
!define MUI_FINISHPAGE_RUN
!define MUI_FINISHPAGE_RUN_NOTCHECKED
!define MUI_FINISHPAGE_RUN_TEXT "Launch ${PRODUCT_NAME}"
!define MUI_FINISHPAGE_RUN_FUNCTION "LaunchLink"
!define MUI_FINISHPAGE_SHOWREADME_NOTCHECKED
!define MUI_FINISHPAGE_SHOWREADME "$INSTDIR\Readme.txt"

;Start Menu Folder Page Configuration
!define MUI_STARTMENUPAGE_DEFAULTFOLDER "${PRODUCT_NAME}"
!define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKCU" 
!define MUI_STARTMENUPAGE_REGISTRY_KEY "${PRODUCT_DIR_REGKEY}" 
!define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"

;--------------------------------
;Pages
!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_COMPONENTS
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_STARTMENU "StartMenuPage" $StartMenuFolder
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_WELCOME
;!insertmacro MUI_UNPAGE_COMPONENTS
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_UNPAGE_FINISH

;--------------------------------
;Languages
!insertmacro MUI_LANGUAGE "English"

Section "Install ${PRODUCT_NAME}" SEC01

	SectionIn RO
	
	SetOverwrite try
	


	ReadRegDWORD $0 HKLM "SOFTWARE\Microsoft\NET Framework Setup\NDP\v4\Client" Install
	IntOp $8 $0 & 1
	IntCmp $8 1 ContinueInstall1 noDotNet ContinueInstall1 

	noDotNet: 
		MessageBox MB_YESNO|MB_ICONQUESTION \
		"Warning! This program requires .Net Framwork 4.0 \
		Do you want to download and install it right now? \
		(If you click No then probably will get error message instead working program )" \
		IDYES installDotNet \
		IDNO ContinueInstall1
	installDotNet:
		SetOutPath "$PLUGINSDIR" 
		File "dotNetFx40_Full_setup.exe" 
		ExecWait "$PLUGINSDIR\\dotNetFx40_Full_setup.exe /norestart /passive /showfinalerror" 
	ContinueInstall1:


	SetOutPath "$INSTDIR"
	File "DevExpress.Data.v11.2.dll"
	File "DevExpress.Printing.v11.2.Core.dll"
	File "DevExpress.Utils.v11.2.dll"
	File "DevExpress.XtraBars.v11.2.dll"
	File "DevExpress.XtraEditors.v11.2.dll"
	File "DevExpress.XtraGrid.v11.2.dll"
	File "DevExpress.XtraLayout.v11.2.dll"
	File "DevExpress.XtraNavBar.v11.2.dll"
	File "DevExpress.XtraTreeList.v11.2.dll"
	File "DevExpress.XtraVerticalGrid.v11.2.dll"
	File "Ionic.Zip.dll"
	File "readme.txt"
	File "filelist.csv"
	File "stats_metadata.csv"
	
	File "WarzoneStatEditor (version 5).exe"
	
	SetOutPath "$INSTDIR\de"
	File "de\DevExpress.Data.v11.2.resources.dll"
	File "de\DevExpress.Utils.v11.2.resources.dll"
	File "de\DevExpress.XtraBars.v11.2.resources.dll"
	File "de\DevExpress.XtraEditors.v11.2.resources.dll"
	File "de\DevExpress.XtraGrid.v11.2.resources.dll"
	File "de\DevExpress.XtraNavBar.v11.2.resources.dll"
	File "de\DevExpress.XtraTreeList.v11.2.resources.dll"
	File "de\DevExpress.XtraVerticalGrid.v11.2.resources.dll"
	
	SetOutPath "$INSTDIR\es"
	File "es\DevExpress.Data.v11.2.resources.dll"
	File "es\DevExpress.Utils.v11.2.resources.dll"
	File "es\DevExpress.XtraBars.v11.2.resources.dll"
	File "es\DevExpress.XtraEditors.v11.2.resources.dll"
	File "es\DevExpress.XtraGrid.v11.2.resources.dll"
	File "es\DevExpress.XtraNavBar.v11.2.resources.dll"
	File "es\DevExpress.XtraTreeList.v11.2.resources.dll"
	File "es\DevExpress.XtraVerticalGrid.v11.2.resources.dll"
	
	SetOutPath "$INSTDIR\ja"
	File "ja\DevExpress.Data.v11.2.resources.dll"
	File "ja\DevExpress.Utils.v11.2.resources.dll"
	File "ja\DevExpress.XtraBars.v11.2.resources.dll"
	File "ja\DevExpress.XtraEditors.v11.2.resources.dll"
	File "ja\DevExpress.XtraGrid.v11.2.resources.dll"
	File "ja\DevExpress.XtraNavBar.v11.2.resources.dll"
	File "ja\DevExpress.XtraTreeList.v11.2.resources.dll"
	File "ja\DevExpress.XtraVerticalGrid.v11.2.resources.dll"
	
	SetOutPath "$INSTDIR\Resources"
	File "Resources\eye.png"
	File "Resources\Icon1.ico"
	File "Resources\warzone2100.png"
	
	SetOutPath "$INSTDIR\ru"
	File "ru\DevExpress.Data.v11.2.resources.dll"
	File "ru\DevExpress.Utils.v11.2.resources.dll"
	File "ru\DevExpress.XtraBars.v11.2.resources.dll"
	File "ru\DevExpress.XtraEditors.v11.2.resources.dll"
	File "ru\DevExpress.XtraGrid.v11.2.resources.dll"
	File "ru\DevExpress.XtraNavBar.v11.2.resources.dll"
	File "ru\DevExpress.XtraTreeList.v11.2.resources.dll"
	File "ru\DevExpress.XtraVerticalGrid.v11.2.resources.dll"
	
	
	;Startmenu shortcuts
	!insertmacro MUI_STARTMENU_WRITE_BEGIN "StartMenuPage"
		CreateDirectory "$SMPROGRAMS\$StartMenuFolder"
		CreateShortCut "$SMPROGRAMS\$StartMenuFolder\${PRODUCT_NAME}.lnk" "$INSTDIR\WarzoneStatEditor (version 5).exe"
		CreateShortCut "$SMPROGRAMS\$StartMenuFolder\Uninstall ${PRODUCT_NAME}.lnk" "$INSTDIR\Uninst.exe"
	!insertmacro MUI_STARTMENU_WRITE_END
	
	CreateShortCut "$DESKTOP\Warzone Stat Editor.lnk" "$INSTDIR\WarzoneStatEditor (version 5).exe"

	WriteUninstaller "$INSTDIR\uninst.exe"
	WriteRegStr HKLM "${PRODUCT_DIR_REGKEY}" "" "$INSTDIR\WarzoneStatEditor (version 5).exe"
	WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayName" "$(^Name)"
	WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "UninstallString" "$INSTDIR\uninst.exe"
	WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayIcon" "$INSTDIR\WarzoneStatEditor (version 5).exe"
	WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayVersion" "${PRODUCT_VERSION}"
	
SectionEnd

Section "Register File Exts" SEC02
	;Delete registred extension from previous ClickOnce deployment
	DeleteRegKey HKEY_CLASSES_ROOT ".wz"
	DeleteRegKey HKEY_CLASSES_ROOT ".crab"
	;Register file types
	WriteRegStr HKLM "SOFTWARE\Classes\.wz" "" "WZ2100.MapModFile"
	WriteRegStr HKLM "SOFTWARE\Classes\.crab" "" "WZ2100.StatEditorFile"

	; This is carfully handled so it doesn't collide with some other utils that also handle the .wz extension
	WriteRegStr HKLM "SOFTWARE\Classes\WZ2100.MapModFile" "" "Warzone 2100 map/mod file"
	WriteRegStr HKLM "SOFTWARE\Classes\WZ2100.MapModFile\shell\EditMod" "" "Open in ${PRODUCT_NAME}"
	WriteRegStr HKLM "SOFTWARE\Classes\WZ2100.MapModFile\shell\EditMod\command" "" '"$INSTDIR\WarzoneStatEditor (version 5).exe" "%1"'

	WriteRegStr HKLM "SOFTWARE\Classes\WZ2100.StatEditorFile" "" "Warzone 2100 Stat Editor file"
	WriteRegStr HKLM "SOFTWARE\Classes\WZ2100.StatEditorFile\shell\EditMod" "" "Open in ${PRODUCT_NAME}"
	WriteRegStr HKLM "SOFTWARE\Classes\WZ2100.StatEditorFile\shell\EditMod\command" "" '"$INSTDIR\WarzoneStatEditor (version 5).exe" "%1"'
	
SectionEnd

Function LaunchLink
	Exec '$INSTDIR\WarzoneStatEditor (version 5).exe'
FunctionEnd

; Section descriptions
!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${SEC01} "The main program."
  !insertmacro MUI_DESCRIPTION_TEXT ${SEC02} "Register .wz files to open in this program."
!insertmacro MUI_FUNCTION_DESCRIPTION_END


Section Uninstall
	Delete "$INSTDIR\uninst.exe"
	Delete "$INSTDIR\WarzoneStatEditor (version 5).exe"
	Delete "$INSTDIR\filelist.csv"
	Delete "$INSTDIR\stats_metadata.csv"
	Delete "$INSTDIR\readme.txt"
	
	Delete "$INSTDIR\DevExpress.XtraVerticalGrid.v11.2.dll"
	Delete "$INSTDIR\DevExpress.XtraTreeList.v11.2.dll"
	Delete "$INSTDIR\DevExpress.XtraNavBar.v11.2.dll"
	Delete "$INSTDIR\DevExpress.XtraGrid.v11.2.dll"
	Delete "$INSTDIR\DevExpress.XtraLayout.v11.2.dll"
	Delete "$INSTDIR\DevExpress.XtraEditors.v11.2.dll"
	Delete "$INSTDIR\DevExpress.XtraBars.v11.2.dll"
	Delete "$INSTDIR\DevExpress.Utils.v11.2.dll"
	Delete "$INSTDIR\DevExpress.Printing.v11.2.Core.dll"
	Delete "$INSTDIR\DevExpress.Data.v11.2.dll"
	Delete "$INSTDIR\Ionic.Zip.dll"
	
	Delete "$INSTDIR\ru\DevExpress.XtraVerticalGrid.v11.2.resources.dll"
	Delete "$INSTDIR\ru\DevExpress.XtraTreeList.v11.2.resources.dll"
	Delete "$INSTDIR\ru\DevExpress.XtraNavBar.v11.2.resources.dll"
	Delete "$INSTDIR\ru\DevExpress.XtraGrid.v11.2.resources.dll"
	Delete "$INSTDIR\ru\DevExpress.XtraEditors.v11.2.resources.dll"
	Delete "$INSTDIR\ru\DevExpress.XtraBars.v11.2.resources.dll"
	Delete "$INSTDIR\ru\DevExpress.Utils.v11.2.resources.dll"
	Delete "$INSTDIR\ru\DevExpress.Data.v11.2.resources.dll"
	
	Delete "$INSTDIR\Resources\warzone2100.png"
	Delete "$INSTDIR\Resources\Icon1.ico"
	Delete "$INSTDIR\Resources\eye.png"
	
	Delete "$INSTDIR\ja\DevExpress.XtraVerticalGrid.v11.2.resources.dll"
	Delete "$INSTDIR\ja\DevExpress.XtraTreeList.v11.2.resources.dll"
	Delete "$INSTDIR\ja\DevExpress.XtraNavBar.v11.2.resources.dll"
	Delete "$INSTDIR\ja\DevExpress.XtraGrid.v11.2.resources.dll"
	Delete "$INSTDIR\ja\DevExpress.XtraEditors.v11.2.resources.dll"
	Delete "$INSTDIR\ja\DevExpress.XtraBars.v11.2.resources.dll"
	Delete "$INSTDIR\ja\DevExpress.Utils.v11.2.resources.dll"
	Delete "$INSTDIR\ja\DevExpress.Data.v11.2.resources.dll"
	
	Delete "$INSTDIR\es\DevExpress.XtraVerticalGrid.v11.2.resources.dll"
	Delete "$INSTDIR\es\DevExpress.XtraTreeList.v11.2.resources.dll"
	Delete "$INSTDIR\es\DevExpress.XtraNavBar.v11.2.resources.dll"
	Delete "$INSTDIR\es\DevExpress.XtraGrid.v11.2.resources.dll"
	Delete "$INSTDIR\es\DevExpress.XtraEditors.v11.2.resources.dll"
	Delete "$INSTDIR\es\DevExpress.XtraBars.v11.2.resources.dll"
	Delete "$INSTDIR\es\DevExpress.Utils.v11.2.resources.dll"
	Delete "$INSTDIR\es\DevExpress.Data.v11.2.resources.dll"
	
	Delete "$INSTDIR\de\DevExpress.XtraVerticalGrid.v11.2.resources.dll"
	Delete "$INSTDIR\de\DevExpress.XtraTreeList.v11.2.resources.dll"
	Delete "$INSTDIR\de\DevExpress.XtraNavBar.v11.2.resources.dll"
	Delete "$INSTDIR\de\DevExpress.XtraGrid.v11.2.resources.dll"
	Delete "$INSTDIR\de\DevExpress.XtraEditors.v11.2.resources.dll"
	Delete "$INSTDIR\de\DevExpress.XtraBars.v11.2.resources.dll"
	Delete "$INSTDIR\de\DevExpress.Utils.v11.2.resources.dll"
	Delete "$INSTDIR\de\DevExpress.Data.v11.2.resources.dll"
	
	RMDir "$INSTDIR\ru"
	RMDir "$INSTDIR\Resources"
	RMDir "$INSTDIR\ja"
	RMDir "$INSTDIR\es"
	RMDir "$INSTDIR\de"
	RMDir "$INSTDIR"

	!insertmacro MUI_STARTMENU_GETFOLDER "StartMenuPage" $StartMenuFolder
	
	Delete "$SMPROGRAMS\$StartMenuFolder\Uninstall ${PRODUCT_NAME}.lnk"
	Delete "$SMPROGRAMS\$StartMenuFolder\${PRODUCT_NAME}.lnk"
	RMDir "$SMPROGRAMS\$StartMenuFolder"
	
	DeleteRegKey /ifempty HKCU "${PRODUCT_DIR_REGKEY}"
	DeleteRegKey HKLM "SOFTWARE\Classes\WZ2100.MapModFile\shell\EditMod"
	DeleteRegKey ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}"
	
	; remove the desktop shortcut icon
	Delete "$DESKTOP\${PRODUCT_NAME}.lnk"
  
  
SectionEnd