﻿/*
	This file is part of Warzone Stat Editor by crab
    Copyright © 2011, 2012 Vladimir Salnikov
 
	Warzone Stat Editor by crab is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	Warzone Stat Editor by crab is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Warzone Stat Editor by crab; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DevExpress.XtraEditors;
using System.IO;
using Ionic.Zip;
using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid;
using System.Deployment.Application;
using System.Text.RegularExpressions;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraSplashScreen;
using System.Threading;

namespace WZStats
{
    public partial class MainForm : DevExpress.XtraEditors.XtraForm, VirtualForm
    {
        public Stats OpenedStats;
        public Stats OriginalWzStats;
        public string OpenedFile = "";
        public string SelectedFile = "";
        SynchronizationContext synch_context;
        public MainForm(string selected_file_path)
        {
            InitializeComponent();
            SelectedFile = selected_file_path;
            synch_context = SynchronizationContext.Current ?? new SynchronizationContext();
            OriginalWzStats = null;
        }

        private void linkWeapons_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("weapons.txt");
        }

        public void EditFields_CommonTableForm_OnClosing(object sender, FormClosingEventArgs e)
        {
            MessageBox.Show(" Removed EditFields_CommonTableForm_OnClosing, sorry");
            //CommonTableForm ctf = sender as CommonTableForm;
            //if (ctf.DataWasChanged)
            //{
            //    DialogResult dr = MessageBox.Show("Data was changed. Save? this action cannot be undone", "Warning!", MessageBoxButtons.YesNoCancel);
            //    if (dr == DialogResult.Yes)
            //    {
            //        OpenedStats.statfields.Save("stats_metadata.csv");
            //    }
            //    else if (dr == DialogResult.No)
            //    {

            //    }
            //    else if (dr == DialogResult.Cancel)
            //    {
            //        e.Cancel = true;
            //    }
            //}
        }

        private void linkEditFields_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            MessageBox.Show(" Removed linkEditFields_ItemClick, sorry");
            //CommonTableForm ctf = new CommonTableForm();
            //ctf.gridControl1.DataSource = OpenedStats.statfields.data;
            //ctf.Text = "Stat field editor (dont touch contents!)";
            //Label l = new Label();
            //ctf.ShowInTaskbar = true;
            //ctf.FormClosing += EditFields_CommonTableForm_OnClosing;
            //ctf.Show();
        }

        public void Log(string msg)
        {
            File.AppendAllText("Log.txt", msg);

        }

        private string FormCaptionEmpty()
        {
            //string version = "";
            //if (ApplicationDeployment.IsNetworkDeployed) //deployment version which shown in project publish properties
            //{
            //    version = ApplicationDeployment.CurrentDeployment.CurrentVersion.ToString(4);
            //}
            //else//Assembly version from AssembleInfo.cs
            //{
            //    version = "DEBUG: " + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString(4);
            //}
            string version = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString(4);
            return "Warzone stat editor v.5.1 [" + version + "] ";
        }

        public void OpenFile(string filename="")
        {
            if (!ProcCloseFile())
                return;
            try
            {
                if(filename == "")
                    if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                        filename = openFileDialog1.FileName;
                if(filename != "")
                {
                    OpenedStats = new Stats("filelist.csv", "stats_metadata.csv", this);
                    memoLog.Text += "\r\nLoading file " + filename + " Wait...";
                    OpenedStats.Load(filename);
                    OpenedFile = filename;
                    this.Text = FormCaptionEmpty() + " " + Path.GetFileName(filename);
                    memoLog.Text += "\r\nLoaded WZ-file " + Path.GetFileName(filename);
                }
                navBarControl1.Enabled = true;
                labelCurFile.Text = "Opened file: " + filename;
            }
            catch (Exception err)
            {
                MessageBox.Show("Open file operation failed with error: " + err.Message);
            }
        }

        public void LoadOriginalStatsInThread(object wzpath)
        {
            string wz_dir = (string)wzpath;
            if (Directory.Exists(wz_dir))
            {
                Stats tmp_stats = new Stats("filelist.csv", "stats_metadata.csv", this);
                string errtxt = "";
                memoLog.Invoke((MethodInvoker)delegate { memoLog.Text += "\r\nTrying load original wz stats.."; });
                if (TryLoadOriginalStats(wz_dir, ref tmp_stats, out errtxt))
                {
                    if (wz_dir == Properties.Settings.Default.WZdir) //check, because it can be changed by user while we loading stats
                        try
                        {
                            synch_context.Send(s =>
                            {
                                OriginalWzStats = tmp_stats;
                                memoLog.Text += "\r\nOriginal wz stats was loaded for purpose";
                            }, null);
                        }
                        catch (InvalidAsynchronousStateException exp)
                        {
                            //catch synch expection to hide error messages to user
                            //this error occures if user closed program before we end
                        }
                }
                else
                {
                    memoLog.Invoke((MethodInvoker)delegate { memoLog.Text += "\r\nFailed load original wz stats"; });
                }
            }
        }

        public void StartLoadingOriginalStats(string wzpath)
        {
            OriginalStatsLoaderThread = new Thread(LoadOriginalStatsInThread);
            OriginalStatsLoaderThread.Priority = ThreadPriority.BelowNormal;
            OriginalStatsLoaderThread.Start(wzpath);
        }

        private void barButtonItem3_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            OpenFile();
        }
        Thread OriginalStatsLoaderThread;
        private void MainForm_Load(object sender, EventArgs e)
        {
            System.IO.Directory.SetCurrentDirectory(Path.GetDirectoryName(Application.ExecutablePath));
            while (!Directory.Exists(Properties.Settings.Default.WZdir) || !CheckWzDir(Properties.Settings.Default.WZdir))
            {
                if (MessageBox.Show("Please specify path to Warzone install directory", "", MessageBoxButtons.YesNo)
                    == System.Windows.Forms.DialogResult.Yes)
                {
                    if (folderBrowserDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        Properties.Settings.Default.PropertyValues["WZdir"].PropertyValue = folderBrowserDialog1.SelectedPath;
                        Properties.Settings.Default.Save();
                    }
                }
                else
                    break;
            }
            memoLog.Text += "\r\nReading list of files from filelist.csv...";
            try
            {
                this.Text = FormCaptionEmpty();
                labelWZdir.Text = "WZ directory path:" + Properties.Settings.Default.WZdir;
                if (SelectedFile != "")
                {
                    if (File.Exists(SelectedFile))
                        OpenFile(SelectedFile);
                }
                else
                {
                    OpenedStats = new Stats("filelist.csv", "stats_metadata.csv", this);
                }
                StartLoadingOriginalStats(Properties.Settings.Default.WZdir);
            }
            catch (Exception eer)
            {
                MessageBox.Show(eer.Message);
            }

        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = !ProcCloseFile();
        }

        bool CheckFileLoaded(string filename)
        {
            if (OpenedStats.FileList.ContainsKey(filename))
            {
                if (OpenedStats.FileList[filename].Loaded)
                {
                    return true;
                }
                else
                {
                    MessageBox.Show(filename + " was not loaded. You can use load functions or open another wz-file");
                }
            }
            else
            {
                MessageBox.Show("File " + filename + " does not exists in file table filelist.csv");
            }
            return false;
        }

        void AdjustGrid(Stats stats, string filename, DevExpress.XtraGrid.Views.Grid.GridView grid)
        {
            DataRow[] fields = stats.statfields.data.Select("filename = '" + filename +"'");
            int gridwidth = 100;
            foreach(DataRow field in fields)
            {
                int value = 0;
                if (Int32.TryParse(field["ColumnVisibleIndex"].ToString(), out value))
                {
                    grid.Columns[field["Caption"].ToString()].VisibleIndex = value;
                }
            
                value = 0;
                if (Int32.TryParse(field["ColumnWidth"].ToString(), out value))
                {
                    gridwidth += value;
                    grid.Columns[field["Caption"].ToString()].Resize(value);
                    if (value == 0)
                        grid.Columns[field["Caption"].ToString()].Visible = false;
                }

                DevExpress.XtraEditors.Repository.RepositoryItemTextEdit tx =
                    new DevExpress.XtraEditors.Repository.RepositoryItemTextEdit();
                tx.ReadOnly = true;
                grid.Columns[field["Caption"].ToString()].ColumnEdit = tx;
                
            }
            if (grid.Columns["Name"]!=null)
            {
                gridwidth += 200;
                grid.Columns["Name"].Resize(200);
                grid.Columns["Name"].VisibleIndex = 0;
                Font f = new System.Drawing.Font(grid.Columns["Name"].AppearanceCell.Font.FontFamily,
                    grid.Columns["Name"].AppearanceCell.Font.Size - 1, FontStyle.Bold);
                grid.Columns["Name"].AppearanceCell.Font = f;
            }
            grid.GridControl.Width = gridwidth;
        }


        private void LinkerEngine_RGrid_FocusedRecordChanged(object sender,
            EventArgs e,
            DataView dw, Stats.Linker vstr)
        {
            /* LINKER ENGINE (link one table to another through stage table)
             * Event when user click row in look-up table 
             * occurs on changind\adding values in stage table
             * L - lookuptable, R - table where we should make list of values, Link - table which contain list of value for all elemets of table R
             * */
            DataNavigator nav = sender as DataNavigator;
            DataTable dt = nav.DataSource as DataTable;
            if(dt == null)
                return;
            int pos = nav.Position;
            if (pos >= 0 && pos < dt.Rows.Count)
            {
                DataRow row = dt.Rows[pos];
                dw.RowFilter = vstr.link_rkey + " = '" + row[vstr.rkey].ToString() + "'";
            }
            else
            {
                dw.RowFilter = vstr.link_rkey + " = ''";
            }
        }


        private void Lister_OpenListItem(
            object sender, 
            DevExpress.XtraEditors.Controls.ButtonPressedEventArgs e,
            Stats.ListFromFile lister, //Link properties
            EditorForm form)
        {   
            if ((string)e.Button.Tag == "eye")
            {
                GridLookUpEdit gle = sender as GridLookUpEdit;
                OpenStatEditor_forCommonFile(lister.lfile, (string)gle.EditValue);
            }
        }

        private void LinkSelect_RowClick(
            object sender, //L-grid
            DevExpress.XtraGrid.Views.Grid.RowClickEventArgs e, 
            ButtonEdit bedit, //button which call ls (in cell)
            Stats.Linker linker, //Link properties
            ListSelect ls, //lookup form for L-grid
            DevExpress.XtraGrid.Views.Grid.GridView TargetGrid, //Link grid
            DevExpress.XtraGrid.Views.Grid.GridView MainRGrid, //R-grid)
            EditorForm form,
            int selected_lfile)
        {
            /* LINKER ENGINE (link one table to another through stage table)
             * Event when user click row in look-up table 
             * occurs on changind\adding values in stage table
             * L - lookuptable, R - table where we should make list of values, Link - table which contain list of value for all elemets of table R
             * */
            DevExpress.XtraGrid.Views.Grid.GridView gview = sender as DevExpress.XtraGrid.Views.Grid.GridView;
            int RightRowIndex = form.dataNavigator1.Position;
            int LeftIndex = ls.dataNavigator1.Position;
            if (e.Clicks > 1)
            {
                DataView link_dw = TargetGrid.GridControl.DataSource as DataView;
                DataTable LookupLTable = gview.GridControl.DataSource as DataTable; //L-grid
                DataTable MainRTable = MainRGrid.GridControl.DataSource as DataTable;
                int dw_row = TargetGrid.GetDataSourceRowIndex(TargetGrid.FocusedRowHandle);
                if (TargetGrid.IsNewItemRow(TargetGrid.FocusedRowHandle)) //add new row in link table
                {  
                    DataRow new_row = link_dw.Table.NewRow();
                    new_row[linker.link_lkey] = LookupLTable.Rows[LeftIndex][linker.lkey].ToString();
                    new_row[linker.link_rkey] = MainRTable.Rows[RightRowIndex][linker.rkey];
                    if (linker.linkTypeFieldName != "")
                        new_row[linker.linkTypeFieldName] = linker.linkTypes[selected_lfile];
                    foreach (string showncolumn in linker.linkcolumns)
                        if (LookupLTable.Columns.Contains(showncolumn) && link_dw.Table.Columns.Contains(showncolumn))
                            new_row[showncolumn] = LookupLTable.Rows[LeftIndex][showncolumn].ToString();
                    link_dw.Table.Rows.Add(new_row);
                }
                else //update left key in link table
                {
                    link_dw[dw_row][linker.link_lkey] = LookupLTable.Rows[LeftIndex][linker.lkey].ToString();
                    if (linker.linkTypeFieldName != "") //type of link 9requred for result components)
                        link_dw[dw_row][linker.link_lkey] = linker.linkTypes[selected_lfile];
                    foreach (string showncolumn in linker.linkcolumns)
                        if(LookupLTable.Columns.Contains(showncolumn))
                            link_dw[dw_row][showncolumn] = LookupLTable.Rows[LeftIndex][showncolumn].ToString();
                }
                MainRTable.Rows[RightRowIndex][linker.CountColumnName] = link_dw.Count;
                TargetGrid.CloseEditor();
                ls.Close();
            }
        }

        private void LinkEngine_SelectedLeftFile(
           object sender, //L-grid
           DevExpress.XtraGrid.Views.Grid.RowClickEventArgs e,
           ButtonEdit bedit, //button which call ls (in cell)
           Stats.Linker linker, //Link properties
           ListSelect LeftFileList, //lookup form for L-grid
           DevExpress.XtraGrid.Views.Grid.GridView TargetGrid, //Link grid
           DevExpress.XtraGrid.Views.Grid.GridView MainRGrid, //R-grid
           EditorForm form)
        {
            /* LINKER ENGINE (link one table to another through stage table)
             * Event when user click row in look-up table 
             * occurs on changind\adding values in stage table
             * L - lookuptable, R - table where we should make list of values, Link - table which contain list of value for all elemets of table R
             * */
            if (e.Clicks > 1)
            {
                string lfile = "";
                int selected_file_index = 0;
                if (LeftFileList != null)
                {
                    selected_file_index = LeftFileList.dataNavigator1.Position;
                    lfile = linker.lfiles[selected_file_index];
                }
                else
                    lfile = linker.lfiles[selected_file_index];

                DataTable list_source;
                if (form.stats.FileList[lfile].Loaded)
                {
                    list_source = form.stats.FileList[lfile].data;
                }
                else if (OriginalWzStats.FileList[lfile].Loaded) 
                {
                    list_source = OriginalWzStats.FileList[lfile].data;
                }
                else
                {
                    MessageBox.Show("Selected file was not loaded.");
                    return;
                }
                ListSelect ls = new ListSelect();
                ls.gridControl1.DataSource = list_source;
                AdjustGrid(form.stats, lfile, ls.gridView1);
                ls.gridView1.RowClick += delegate(object sender2, DevExpress.XtraGrid.Views.Grid.RowClickEventArgs e2)
                { LinkSelect_RowClick(sender2, e2, sender as ButtonEdit, linker, ls, TargetGrid, MainRGrid, form, selected_file_index); };
                if (LeftFileList != null) //null if we skip select left file
                    LeftFileList.Close();
                ls.ShowDialog();
            }
        }



        private void SelectValueInList_ButtonClick(object sender,
            DevExpress.XtraEditors.Controls.ButtonPressedEventArgs e, 
            Stats.Linker linker,
            DevExpress.XtraGrid.Views.Grid.GridView TargetGrid,
            DevExpress.XtraGrid.Views.Grid.GridView MainRGrid,
            EditorForm form,
            bool HaveNames)
        {
            /* LINKER ENGINE (link one table to another through stage table)
             * Event when user click row in look-up table 
             * occurs on changind\adding values in stage table
             * L - lookuptable, R - table where we should make list of values, Link - table which contain list of value for all elemets of table R
             * */
            if ((string)e.Button.Tag == "eye") //open linked item in new edit window
            {
                int row_h = TargetGrid.FocusedRowHandle;
                if (row_h != GridControl.InvalidRowHandle)
                {
                    DataRow link_row = TargetGrid.GetDataRow(row_h);
                    if (link_row != null)
                    {
                        string key_value = (string)link_row[linker.link_lkey];
                        foreach (string filename in linker.lfiles)
                        {
                            if (form.stats.FileList[filename].Loaded)
                            {
                                DataTable dt = form.stats.FileList[filename].data;
                                if (dt.Columns.Contains(linker.lkey))
                                {
                                    DataRow[] rows = form.stats.FileList[filename].data.Select
                                                    ("[" + linker.lkey + "] = '" + key_value + "'");
                                    if (rows.Length > 0)
                                    {
                                        OpenStatEditor_forCommonFile(filename, key_value);
                                    }
                                }
                            }
                        }
                    }
                }
            }else if(!form.read_only) 
            {
            //add or change current linked item
                if (linker.lfiles.Length == 1)
                {
                    DevExpress.Utils.DXMouseEventArgs tmp
                        = new DevExpress.Utils.DXMouseEventArgs(System.Windows.Forms.MouseButtons.Left, 2, 1, 1, 0);
                    DevExpress.XtraGrid.Views.Grid.RowClickEventArgs en = new DevExpress.XtraGrid.Views.Grid.RowClickEventArgs(tmp,0);
                    LinkEngine_SelectedLeftFile(sender, en, sender as ButtonEdit, linker, null, TargetGrid, MainRGrid, form);
                }else if(linker.lfiles.Length > 1 )
                {
                    ListSelect select_file = new ListSelect();
                    select_file.Width = 300;
                    select_file.Height = 300;
                    select_file.gridControl1.DataSource = linker.lfiles;
                    select_file.gridView1.RowClick += delegate(object sender2, DevExpress.XtraGrid.Views.Grid.RowClickEventArgs e2)
                    { LinkEngine_SelectedLeftFile(sender2, e2, sender as ButtonEdit, linker, select_file, TargetGrid, MainRGrid,form); };
                    select_file.ShowDialog();
                }
            }

        }


        private void DeleteRow_Click(object sender, EventArgs e, DevExpress.XtraGrid.Views.Grid.GridView TargetGrid)
        {
            TargetGrid.DeleteRow(TargetGrid.FocusedRowHandle);
        }

        private void LinkEngine_DeleteLinkRow(object sender, DataRowChangeEventArgs e, 
            EditorForm form, string Rfile, Stats.Linker linker)
        {
            
            int pos = form.dataNavigator1.Position; //position in Main Table (current stat file)
            if ((form.dataNavigator1.DataSource as DataTable) == null)
                return;
            if(pos <0 || pos >= (form.dataNavigator1.DataSource as DataTable).Rows.Count)
            {
                return;
            }
            DataTable link_dt = form.stats.FileList[linker.link_file].data;
            DataTable r_dt = form.stats.FileList[Rfile].data;
            if(r_dt.Rows[pos][linker.CountColumnName] != null)
            {
                DataRow[] link_rows = link_dt.Select("[" + linker.link_rkey + "] = '" + r_dt.Rows[pos][linker.rkey].ToString() + "'");
                r_dt.Rows[pos][linker.CountColumnName] = link_rows.Length;
            }
        }

        void AdjustEditorVGrid(Stats stats, string statfile, DevExpress.XtraVerticalGrid.VGridControl vgrid, EditorForm editform)
        {
            //Name column, can be empty
            if (stats.FileList[statfile].data.Columns.Contains("Name"))
            {
                DevExpress.XtraVerticalGrid.Rows.EditorRow row =
                         new DevExpress.XtraVerticalGrid.Rows.EditorRow();
                row.Properties.Caption = "Name";
                row.Properties.FieldName = "Name";
                vgrid.Rows.Add(row);
            }
            string curr_category = "";
            DataRow[] fields = stats.statfields.data.Select(
                "filename = '" + statfile + "'", "CategoryVisibleIndex ASC");
            foreach (DataRow field in fields)
            {
                bool RowEditorSetted = false;
                if (curr_category != field["CategoryName"].ToString())
                {
                    curr_category = field["CategoryName"].ToString();
                    DevExpress.XtraVerticalGrid.Rows.CategoryRow cat_row
                        = new DevExpress.XtraVerticalGrid.Rows.CategoryRow(curr_category);
                    vgrid.Rows.Add(cat_row);

                }

                DevExpress.XtraVerticalGrid.Rows.EditorRow row =
                    new DevExpress.XtraVerticalGrid.Rows.EditorRow();
                row.Properties.Caption = field["Caption"].ToString();
                row.Properties.FieldName = field["Caption"].ToString();
                row.Name = field["Caption"].ToString();
                bool read_only = editform.read_only;
                bool skip_row = false;
                //1. Check Expression type (if defined)
                if ((StatMetadata.ExType)field["ExType"] == StatMetadata.ExType.LinkTable)
                {
                    read_only = true;
                    skip_row = true;
                }
                else if((StatMetadata.ExType)field["ExType"] == StatMetadata.ExType.List)
                {
                    if(field["Expression"].ToString().Trim() != "")
                    {
                        DevExpress.XtraEditors.Repository.RepositoryItemComboBox cbox =
                            new DevExpress.XtraEditors.Repository.RepositoryItemComboBox();
                        cbox.ReadOnly = read_only;
                        row.Properties.RowEdit = cbox;
                        cbox.Items.AddRange(field["Expression"].ToString().Trim().Split('|'));
                        RowEditorSetted = true;
                    }
                }
                else if ((StatMetadata.ExType)field["ExType"] == StatMetadata.ExType.ListFromFile)
                {
                    Stats.ListFromFile lister = new Stats.ListFromFile((string)field["Expression"]);
                    if (stats.FileList.ContainsKey(lister.lfile))
                    {
                        DataTable ltable = null;
                        if (stats.FileList[lister.lfile].Loaded)
                        {
                            ltable = stats.FileList[lister.lfile].data;
                        }else if(OriginalWzStats != null)
                        {
                            if((OriginalWzStats.FileList[lister.lfile].Loaded))
                            {
                                ltable = OriginalWzStats.FileList[lister.lfile].data;
                            }
                        }
                        if(ltable != null)
                        {
                            DevExpress.XtraEditors.Repository.RepositoryItemGridLookUpEdit be =
                               // vgrid.RepositoryItems["leSelectFromFile"] as DevExpress.XtraEditors.Repository.RepositoryItemGridLookUpEdit;
                                new DevExpress.XtraEditors.Repository.RepositoryItemGridLookUpEdit();

                            be.DataSource = ltable;
                            be.ValueMember = lister.lKeyColumn;
                            if(ltable.Columns.Contains(lister.lNameColumn))
                                be.DisplayMember = lister.lNameColumn;
                            else
                                be.DisplayMember = lister.lKeyColumn;
                            be.PopulateViewColumns();
                            GridView view = be.View as GridView;
                            view.OptionsView.ColumnAutoWidth = false;
                            AdjustGrid(stats, lister.lfile, view);
                            DevExpress.XtraEditors.Controls.EditorButton bt = new DevExpress.XtraEditors.Controls.EditorButton();
                            bt.Image = Properties.Resources.eye;
                            be.Buttons.Add(bt);
                            bt.Kind = DevExpress.XtraEditors.Controls.ButtonPredefines.Glyph;
                            be.ButtonClick += delegate(object sender, DevExpress.XtraEditors.Controls.ButtonPressedEventArgs e)
                            { Lister_OpenListItem(sender, e, lister, editform); };
                            bt.Tag = "eye";
                            be.ReadOnly = read_only;
                            RowEditorSetted = true;
                            row.Properties.RowEdit = be;
                        }
                    }
                }
                //2. check DataType and set editor by datatype
                if (!RowEditorSetted)
                {
                    if ((bool)field["IsInteger"])
                    {
                        DevExpress.XtraEditors.Repository.RepositoryItemSpinEdit spedit =
                            new DevExpress.XtraEditors.Repository.RepositoryItemSpinEdit();
                        spedit.SpinStyle = DevExpress.XtraEditors.Controls.SpinStyles.Horizontal;
                        spedit.ReadOnly = read_only;
                        row.Properties.RowEdit = spedit;
                    }
                    else
                    {
                        DevExpress.XtraEditors.Repository.RepositoryItemTextEdit txedit =
                            new DevExpress.XtraEditors.Repository.RepositoryItemTextEdit();
                        txedit.ReadOnly = read_only;
                        row.Properties.RowEdit = txedit;
                    }
                }
                //row.Properties.ReadOnly = read_only;
                if(!skip_row)
                    vgrid.Rows.Add(row);
            }
        }

        void AdjustLinkedTables(Stats stats, string filename, EditorForm editform)
        {
            DevExpress.XtraVerticalGrid.VGridControl pgrid = editform.vGridControl1;
            DataRow[] fields = stats.statfields.data.Select("filename = '" + filename + "'", "CategoryVisibleIndex ASC");
            List<Control> GridControlsList = new List<Control>(20);
            foreach (DataRow field in fields)
            {
                if ((StatMetadata.ExType)field["ExType"] == StatMetadata.ExType.LinkTable)
                {
                    /* LINKER ENGINE (link one table to another through stage table)
                     * Event when user click row in look-up table 
                     * occurs on changind\adding values in stage table
                     * L - lookuptable, R - table where we should make list of values, Link - table which contain list of value for all elemets of table R
                     * */
                    string value_list = field["Expression"].ToString();
                    if (value_list != "")
                    {
                        //guess column contain number of rows in link-table. it should be adjusted by code

                        Stats.Linker linker = new Stats.Linker(value_list, field["Caption"].ToString());
                        string link_errors = "";
                        if (linker.CheckLink(stats, filename, out link_errors))
                        {
                            //first, update names of L-files (this code works bad if name not specified)
                            foreach (string lfile in linker.lfiles)
                                ConfirmLoadNames(stats, lfile);
                            DataTable link_dt = stats.FileList[linker.link_file].data;

                            link_dt.RowDeleted += delegate(object sender, DataRowChangeEventArgs e)
                            { LinkEngine_DeleteLinkRow(sender, e, editform, filename, linker); };

                            //join all l-file tables into 1 link file (to see names of weapons and bodie in 1 table)
                            bool lfiles_errors = false;
                            foreach (string lfile in linker.lfiles)
                            {
                                if (stats.FileList[lfile].Loaded)
                                {
                                    DataTable tmp_dt = stats.FileList[lfile].data;
                                    linker.Join(stats, linker.link_file, ref link_dt, ref tmp_dt);
                                }
                                else
                                {
                                    lfiles_errors = true;
                                }
                            }
                            lfiles_errors = lfiles_errors || !link_dt.Columns.Contains("Name");//
                            LinkGrid LinkGridControl = new WZStats.LinkGrid(editform.read_only);
                            GridControl vgrid = LinkGridControl.gridControl1;
                            DataView dw = new DataView(link_dt);
                            DevExpress.XtraGrid.Views.Grid.GridView vgridview = LinkGridControl.gridView1;
                            vgrid.DataSource = dw;
                            vgridview.PopulateColumns();
                            LinkGridControl.simpleButton1.Click += delegate(object sender, EventArgs e)
                            { DeleteRow_Click(sender, e, vgridview); };
                            vgridview.ViewCaption = "Contents of " + field["Caption"].ToString();
                            if (editform.read_only)
                                vgridview.OptionsBehavior.AllowAddRows = DevExpress.Utils.DefaultBoolean.False;

                            int first_col_index = 0;
                            int min_col_pos = 1110;
                            for (int i = 0; i < vgridview.Columns.Count; i++)
                            {

                                bool contains = false;
                                int col_pos = -1;
                                if (lfiles_errors) //need show code
                                {
                                    if (vgridview.Columns[i].FieldName == linker.link_lkey)
                                    {
                                        contains = true;
                                    }
                                }
                                foreach (string col in linker.linkcolumns)
                                {
                                    col_pos++;
                                    if (vgridview.Columns[i].FieldName == col)
                                    {
                                        contains = true;
                                        break;
                                    }

                                }
                                if (contains)
                                {
                                    DevExpress.XtraEditors.Repository.RepositoryItemButtonEdit be =
                                        new DevExpress.XtraEditors.Repository.RepositoryItemButtonEdit();
                                    be.TextEditStyle = DevExpress.XtraEditors.Controls.TextEditStyles.DisableTextEditor;
                                    be.ReadOnly = editform.read_only;
                                    DevExpress.XtraEditors.Controls.EditorButton bt = new DevExpress.XtraEditors.Controls.EditorButton();
                                    bt.Image = Properties.Resources.eye;
                                    bt.Tag = "eye";
                                    bt.Kind = DevExpress.XtraEditors.Controls.ButtonPredefines.Glyph;
                                    be.Buttons.Add(bt);
                                    be.ButtonClick +=
                                            delegate(object sender, DevExpress.XtraEditors.Controls.ButtonPressedEventArgs e)
                                            { SelectValueInList_ButtonClick(sender, e, linker, vgridview, editform.gridView1, editform, true); };
                                    vgridview.Columns[i].ColumnEdit = be;
                                    vgridview.Columns[i].VisibleIndex = col_pos;
                                    if (col_pos < min_col_pos)
                                    {
                                        first_col_index = i;
                                        min_col_pos = col_pos;
                                    }
                                }
                                else
                                    vgridview.Columns[i].Visible = false;
                            }
                            if (vgridview.Columns[first_col_index] != null)
                                vgridview.Columns[first_col_index].Width = 140;

                            LinkGridControl.Dock = DockStyle.Top;
                            GridControlsList.Add(LinkGridControl);
                            //editform.ContainerForValueList.Controls.Add(LinkGridControl);
                            LinkGridControl.Width = editform.vGridControl1.Width - 15;
                            LinkGridControl.Height = 80;
                            if (editform.splitContainerControl3.SplitterPosition > 900)
                                editform.splitContainerControl3.SplitterPosition = 470;
                            editform.StartPosition = FormStartPosition.CenterScreen;

                            if (editform.Height < 800)
                            {
                                editform.Height += LinkGridControl.Height;
                            }
                            else
                            {
                                if (editform.splitContainerControl3.SplitterPosition > 200)
                                    editform.splitContainerControl3.SplitterPosition -= LinkGridControl.Height;
                            }
                            int cur_row_h = editform.gridView1.FocusedRowHandle;
                            if (cur_row_h != GridControl.InvalidRowHandle)
                            {
                                if (editform.gridView1.IsNewItemRow(cur_row_h))
                                {
                                    dw.RowFilter = linker.link_rkey + " = ''";
                                }
                                else
                                {
                                    DataRow focused_row = editform.gridView1.GetDataRow(cur_row_h);
                                    dw.RowFilter = linker.link_rkey + " = '" + focused_row[linker.rkey].ToString() + "'";
                                }
                            }
                            editform.dataNavigator1.PositionChanged += delegate(object sender, EventArgs e)
                            { LinkerEngine_RGrid_FocusedRecordChanged(sender, e, dw, linker); };

                        }
                        else // if found linker errors
                        {
                            Label lbl = new Label();
                            lbl.Text = "Cannot show list of [" + field["Caption"].ToString() + "]" +
                                "\r\nError: " + link_errors;
                            lbl.AutoSize = true;
                            GridControlsList.Add(lbl);
                        }
                    }

                }
            }//edit of fields loop
            TableLayoutPanel lc = new TableLayoutPanel();
            lc.Dock = DockStyle.Fill;
            lc.ColumnCount = 1;
            lc.GrowStyle = TableLayoutPanelGrowStyle.AddRows;
            for (int i = 0; i < GridControlsList.Count; i++)
                lc.Controls.Add(GridControlsList[i]);
            editform.ContainerForValueList.Controls.Add(lc);
        }

        void AdjustEditorForm(Stats stats, string filename, EditorForm form, bool HaveNames = false)
        {
            // set up data navigators
            form.gridControl1.DataSource = stats.FileList[filename].data;
            form.dataNavigator1.DataSource = stats.FileList[filename].data;

            //1. set up grid for items
            AdjustGrid(stats, filename, form.gridView1);

            //2. set up SelectionClass from DevExpress to select grid items for comparison functions
            form.SelectionClass = new GridSelectCheckBox(form.gridView1);

            //3. set up vertical grid for item properties
            AdjustEditorVGrid(stats, filename, form.vGridControl1, form);
            form.vGridControl1.FocusedRowChanged += delegate(object sender, DevExpress.XtraVerticalGrid.Events.FocusedRowChangedEventArgs e)
            { PropGrid_FocusedRowChanged(sender, e, form); };
            form.vGridControl1.DataSource = stats.FileList[filename].data; //warning: attach DS should be placed after adjusting grid function(!)

            //4. set up linked tables in propertioes panel
            AdjustLinkedTables(stats, filename, form);

            //5. set up tree control for research tree
            form.treeList1.MouseDoubleClick += ResearchTree_MouseDoubleClick;
  
            //6. set up vertical grid for item comparison
            AdjustEditorVGrid(stats, filename, form.vGridComparison, form);
        }

        private void PropGrid_FocusedRowChanged(object sender, DevExpress.XtraVerticalGrid.Events.FocusedRowChangedEventArgs e, EditorForm form)
        {
            if (e.Row != null)
            {
                DataRow[] rows = form.stats.statfields.data.Select("Caption = '" + e.Row.Properties.FieldName + "'");
                string txt = "";
                if (rows.Length > 0)
                    txt = rows[0]["Description"].ToString();
                form.memoDescr.Text = txt;
            }
        }

        public bool ConfirmLoadNames(Stats stats, string statfile)
        {
            //check if names was loaded
            bool NamesLoaded = false;
            string namesfile = stats.FileList[statfile].NamesFile;
            if (namesfile != "")
            {
                if (stats.FileList.ContainsKey(namesfile))
                    if (stats.FileList[namesfile].Loaded)
                        NamesLoaded = true;
                    else
                    {
                        //if (OriginalWzStats != null)
                        //{
                        //    if (OriginalWzStats.FileList[namesfile].Loaded)
                        //    {
                        //        OpenedStats.FileList[namesfile].names_data = OriginalWzStats.FileList[namesfile].
                        //    }
                        //}
                        if (MessageBox.Show("File with items captions (" + namesfile + ") was not loaded. Do you want to load it now from WZ-folder?", "Warning",
                             MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.Yes)
                        {
                            string mp_filename = Properties.Settings.Default.WZdir + "\\mp.wz";
                            if (File.Exists(mp_filename))
                            {
                                stats.Load(mp_filename, namesfile, true);
                                stats.UpdateNames(namesfile);
                                NamesLoaded = true;
                            }
                            else
                            {
                                MessageBox.Show("Error: File not found: " + mp_filename);
                            }
                            
                        }
                    }
            }
            return NamesLoaded;
        }

        private void ResearchTree_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            DevExpress.XtraTreeList.TreeList tree = sender as DevExpress.XtraTreeList.TreeList;
            DevExpress.XtraTreeList.TreeListHitInfo hi = tree.CalcHitInfo(tree.PointToClient(Control.MousePosition));
            if(hi.Node != null)
            {
                DataRowView clicked_res_row = tree.GetDataRecordByNode(hi.Node) as DataRowView;
                if (clicked_res_row == null) 
                    return;
                OpenStatEditor_forCommonFile("research.txt", (string)clicked_res_row.Row["Code"]);
            }

        }


        public EditorForm OpenStatEditor_forCommonFile(string filename, string KeyFieldValue="")
        {
            if (!OpenedStats.FileList.ContainsKey(filename))
                return null;
            EditorForm result = null;
            if (OpenedStats.FileList[filename].Loaded)
            {
                ShowLoadingPanel();
                EditorForm form = new EditorForm(OpenedStats);
                form.Text = "Stat editor for " + filename;
                bool NamesLoaded = ConfirmLoadNames(OpenedStats, filename);

                AdjustEditorForm(OpenedStats, filename, form, NamesLoaded);
                if (KeyFieldValue != "") //open form positioned on some selected item
                {
                    DataTable dt = OpenedStats.FileList[filename].data;
                    string KeyColumn = dt.Columns[OpenedStats.FileList[filename].KeyColumns[0]].ColumnName;
                    DataRow[] rows = dt.Select(KeyColumn + " = '" + KeyFieldValue + "'");
                    if (rows.Length > 0)
                    {
                        int d = form.gridView1.FocusedRowHandle - form.gridView1.TopRowIndex;
                        form.dataNavigator1.Position = dt.Rows.IndexOf(rows[0]);
                        form.gridView1.TopRowIndex = form.gridView1.FocusedRowHandle - d;
                        form.gridView1.SelectRow(form.gridView1.FocusedRowHandle);
                    }
                }
                form.Show();
                HideLoadingPanel();
                result = form;
            }
            else //load from original stats
            {
                OptionsMessageForm of = new OptionsMessageForm();
                //this.Enabled = false;
#if ! DEBUG
                try
                {
#endif
                    if (of.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        //OPEN STAT FILE READ-ONLY
                        if (of.radioGroup1.SelectedIndex == 0)
                        {
                            ShowLoadingPanel();
                            if (OriginalWzStats == null)
                            {
                                MessageBox.Show("Original WZ stats was not loaded yet. Wait some seconds and try again");
                            }
                            else
                            {
                                EditorForm form = new EditorForm(OriginalWzStats, true);
                                form.Text = "ORIGINAL WARZONE STATS: " + filename;
                                bool NamesLoaded = true;

                                AdjustEditorForm(OriginalWzStats, filename, form, NamesLoaded);
                                if (KeyFieldValue != "") //open form positioned on some selected item
                                {
                                    DataTable dt = OriginalWzStats.FileList[filename].data;
                                    string KeyColumn = dt.Columns[OriginalWzStats.FileList[filename].KeyColumns[0]].ColumnName;
                                    DataRow[] rows = dt.Select(KeyColumn + " = '" + KeyFieldValue + "'");
                                    if (rows.Length > 0)
                                    {
                                        int d = form.gridView1.FocusedRowHandle - form.gridView1.TopRowIndex;
                                        form.dataNavigator1.Position = dt.Rows.IndexOf(rows[0]);
                                        form.gridView1.TopRowIndex = form.gridView1.FocusedRowHandle - d;
                                        form.gridView1.SelectRow(form.gridView1.FocusedRowHandle);
                                    }
                                }
                                form.Show();
                                HideLoadingPanel();
                                result = form;
                            }
                        }
                        //LOAD STAT FILE FROM ORIGINAL WZ
                        else if (of.radioGroup1.SelectedIndex == 1)
                        {
                            string wzpath = Properties.Settings.Default.WZdir;
                            if (CheckWzDir(wzpath))
                            {
                                //try open from mp.wz
                                bool loaded = false;
                                string mpwz = GetMpWzFilename(wzpath);
                                if (File.Exists(mpwz))
                                {
                                    OpenedStats.Load(mpwz, filename, true);
                                    loaded = OpenedStats.FileList[filename].Loaded;
                                    if (loaded)
                                        result =  OpenStatEditor_forCommonFile(filename);
                                }
                                if (!loaded)
                                {
                                    //try open from base.wz
                                    string basewz = GetBaseWzFilename(wzpath);
                                    if (File.Exists(basewz))
                                    {
                                        OpenedStats.Load(basewz, filename, true);
                                        loaded = OpenedStats.FileList[filename].Loaded;
                                        if (OpenedStats.FileList[filename].Loaded)
                                            result = OpenStatEditor_forCommonFile(filename);
                                    }
                                }
                                if(!loaded)
                                    MessageBox.Show("Sorry, i cannot load this stat-file from original WZ stats");
                            }
                        }
                    }
#if ! DEBUG
                }
                finally
                {
#endif
                    //this.Enabled = true;
#if ! DEBUG
                }
#endif
            }
            return result;
        }

        private void linkBodies_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {

            OpenStatEditor_forCommonFile("body.txt");
        }

        private void linkSetWzPath_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            if (Directory.Exists(Properties.Settings.Default.PropertyValues["WZdir"].PropertyValue.ToString()))
                folderBrowserDialog1.SelectedPath = Properties.Settings.Default.PropertyValues["WZdir"].PropertyValue.ToString();
            if (folderBrowserDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                string p = folderBrowserDialog1.SelectedPath;
                if(CheckWzDir(p))
                {
                    Properties.Settings.Default.PropertyValues["WZdir"].PropertyValue = p;
                    Properties.Settings.Default.Save();
                    labelWZdir.Text = "WZ directory path: " + p;
                    StartLoadingOriginalStats(p);
                }else
                {
                    MessageBox.Show("New path was not setted.");
                }
            }
        }

        public string GetMpWzFilename(string Wzdir_path)
        {
            string path = Wzdir_path;
            string slash = "";
            path = path.Trim();
            if (path.Length > 0)
                if (path[path.Length - 1] != '\\')
                    slash = "\\";
           return Path.GetFullPath(path) + slash + "mp.wz";
        }
        public string GetBaseWzFilename(string Wzdir_path)
        {
            string path = Wzdir_path;
            string slash = "";
            path = path.Trim();
            if (path.Length > 0)
                if (path[path.Length - 1] != '\\')
                    slash = "\\";
            return Path.GetFullPath(path) + slash + "base.wz";
        }

        bool CheckWzDir(string path)
        {
            string mpfile = GetMpWzFilename(path);
            string error = "";
            if (!File.Exists(mpfile))
                error += "Warning! File " + mpfile + " not exists\r\n";

            string basefile = GetBaseWzFilename(path);
            if (!File.Exists(basefile))
                error += "Warning! File " + mpfile + " not exists";
            if (error != "")
            {
                MessageBox.Show(error);
                return false;
            }
            return true;
        }

        public bool TryLoadOriginalStats(string wz_dir_path, ref Stats stats, out string errtxt)
        {
            errtxt = "";
            string mpwz, basewz;
            mpwz = GetMpWzFilename(wz_dir_path);
            basewz = GetBaseWzFilename(wz_dir_path);
            if(!File.Exists(mpwz) || !File.Exists(basewz))
                return false;
            try
            {
                string[] files = new string[stats.FileList.Keys.Count];
                stats.FileList.Keys.CopyTo(files, 0);
                //load all possible base files
                foreach (string file in files)
                {
                    stats.Load(basewz, file, true);
                    if (stats.FileList[file].Loaded)
                        stats.FileList[file].NeedSaveInAnyCase = true;
                }
                //then override by mp-files
                foreach (string file in files)
                {
                    stats.Load(mpwz, file, true);
                    if (stats.FileList[file].Loaded)
                        stats.FileList[file].NeedSaveInAnyCase = true;
                }
                //call update names, because some file can be not updated due to load order
                stats.UpdateNames();
            }catch(Exception errs)
            {
                errtxt = errs.Message;
                return false;
            }
            return true;
        }

        private void linkLoadOriginalStats_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            
            string wzpath = Properties.Settings.Default.WZdir;
            if (!CheckWzDir(wzpath))
                return;
            if(MessageBox.Show(
                "All current data will be overriden. Confim load original files??","Warning",MessageBoxButtons.OKCancel)
                == System.Windows.Forms.DialogResult.OK)
            {
                ShowLoadingPanel();
#if ! DEBUG
                try
                {
#endif               
                    string error_text = "";
                    if (TryLoadOriginalStats(wzpath, ref OpenedStats, out error_text))
                    {
                        
                    }
                    else
                    {
                        MessageBox.Show("Loading failed..Error:" + error_text);
                    }
#if ! DEBUG
                }
                finally
                {
#endif
                    HideLoadingPanel();
#if ! DEBUG
                }
#endif
            }
        }


        private void LoadOneFile_FileSelecting(object sender, DevExpress.XtraGrid.Views.Grid.RowClickEventArgs e, ListSelect ls, Stats stats)
        {
            int index = ls.dataNavigator1.Position;
            DevExpress.XtraGrid.Views.Grid.GridView gview = sender as DevExpress.XtraGrid.Views.Grid.GridView;
            if(e.Clicks>1)
            {
                string filename = (gview.GridControl.DataSource as DataTable).Rows[index][0].ToString();
                if (stats.FileList[filename].Loaded)
                    if (MessageBox.Show("File " + filename + " already loaded in current wz-file. Overwrite? if Yes then select wz-file to load from ", "Confirm", MessageBoxButtons.YesNo)
                         == System.Windows.Forms.DialogResult.No)
                    {
                        return;
                    }
                if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    memoLog.Text += "\r\nLoading file " + openFileDialog1.FileName + " Wait...";
                    stats.Load(openFileDialog1.FileName, filename, true);
                    memoLog.Text += "\r\nLoaded WZ-file " + Path.GetFileName(openFileDialog1.FileName);
                    MessageBox.Show("File " + filename + " was loaded succesfully in current opened wz-file");
                }

            }
        }

        private void linkLoadOneFile_ItemClick(object sender2, DevExpress.XtraBars.ItemClickEventArgs e2)
        {
            ListSelect ls = new ListSelect();
            DataTable dt = new DataTable();
            dt.Columns.Add("Filename");
            dt.Columns.Add("Path");
            foreach (string file in OpenedStats.FileList.Keys)
            {
                DataRow row = dt.NewRow();
                row[0] = file;
                row[1] = OpenedStats.FileList[file].path;
                dt.Rows.Add(row);
            }
            ls.gridControl1.DataSource = dt;
            ls.gridView1.PopulateColumns(dt);
            ls.gridView1.Columns[0].Width = 150;
            ls.gridView1.Columns[1].Width = 250;
            ls.gridView1.RowClick += delegate(object sender, DevExpress.XtraGrid.Views.Grid.RowClickEventArgs e)
            { LoadOneFile_FileSelecting(sender, e, ls, OpenedStats); };
            ls.ShowDialog();
        }

        private void navBarControl1_Click(object sender, EventArgs e)
        {

        }

        private void linkPropulsion_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("propulsion.txt");
        }

        private void linkStructures_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("structures.txt");
        }

        private void barButtonItem9_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            SaveCurFile();
        }

        private void barButtonItem4_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            this.Close();
        }

        public void SaveCurFile(bool SaveAs=false)
        {
            /*
             * Case 1: new file
             * Case 2: opened wz file - save as crab file
             * Case 3: SaveAs from wz (crab - crab)
             * Case 4: SaveAs from crab (crab - crab)
             * Case 5: Save opened crab file
             */
#if ! DEBUG
            try
            {
#endif
                if (OpenedFile != "" && !SaveAs && Regex.IsMatch(OpenedFile,@".+\.crab$"))
                {//Case 5: Save opened crab file
                    OpenedStats.Save(OpenedFile);
                }
                else
                {
                    crabSaveDialog.FileName = OpenedStats.wzfileinfo.clear_filename;
                    if (crabSaveDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        //if we save new file we need remove another file under this filename
                        if (File.Exists(crabSaveDialog.FileName))
                            File.Delete(crabSaveDialog.FileName);
                        string sourcefile = "";
                        if (File.Exists(OpenedFile))
                            sourcefile = OpenedFile;
                        OpenedStats.Save(crabSaveDialog.FileName, sourcefile);
                        OpenedFile = crabSaveDialog.FileName;
                        UpdateFormLabels();
                    }

                }
#if ! DEBUG
            }
            catch (Exception err)
            {
                MessageBox.Show("Save file operation failed with error: " + err.Message);
            }
#endif
        }

        public void UpdateFormLabels()
        {
            labelCurFile.Text = "Opened wz-file: " + OpenedFile;
            this.Text = FormCaptionEmpty() + " " + Path.GetFileName(OpenedFile);
        }

        public bool ProcCloseFile()
        {//Ask user to save current file
            if(OpenedStats != null) // only in case error 
                if (OpenedStats.CheckForLoadedFiles())
                {
                    DialogResult dr =
                        MessageBox.Show("Current file possibly was changed. Save changes?", "Warning", 
                        MessageBoxButtons.YesNoCancel);
                    if (dr == DialogResult.Cancel)
                        return false;
                    if (dr == DialogResult.Yes)
                        SaveCurFile();
                }
            return true;
        }

        public void LoadNames(Stats stats)
        {

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                List<string> filestoload = new List<string>();
                string filename = openFileDialog1.FileName;
                //1. collect files fo load with user confirmation
                foreach(string sfile in stats.FileList.Keys)
                {
                    if(stats.FileList[sfile].IsNames)
                    {
                        bool ConfirmLoad = true;
                        if(stats.FileList.ContainsKey(sfile))
                            if(stats.FileList[sfile].Loaded)
                                ConfirmLoad = MessageBox.Show("File " + sfile + " was loaded in current project already. Overwrite?","Confirmation",
                                    MessageBoxButtons.YesNo) == DialogResult.Yes;
                        if (ConfirmLoad)
                            filestoload.Add(sfile); //loading file in current foeach causes error
                    }
                }

                //2. load files and update item's names
                foreach(string f in filestoload)
                {
                    stats.Load(filename, f, true);
                    memoLog.Text += "\r\nFile " + f + " was loaded succesfully";
                }
                stats.UpdateNames();
            }
            
            //end if == DialogResult.OK
        }

        private void linkLoadNames_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            LoadNames(OpenedStats);
        }

        private void linkStatComp_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            StatComparer form = new StatComparer(OriginalWzStats);
            form.Show();
        }

        private void LinkPropulsionDamage_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("weaponmodifier.txt");
        }

        private void linkTerrainTable_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("terraintable.txt");
        }

        private void linkResearch_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("research.txt");
        }

        private void linkFunctions_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("functions.txt");
        }

        private void linkStuctureDamage_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("structuremodifier.txt");
        }

        private void linkContructions_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("construction.txt");
        }

        private void LinkRepair_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("repair.txt");
        }

        private void linkCompiler_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            CompilerForm cf = new CompilerForm();
            cf.Show();
        }

        private void linkWeaponSounds_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("weaponsounds.txt");
        }

        private void linkTemplates_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("templates.txt");
        }

        private void barButtonItem13_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            SaveCurFile(true);
        }

        public void OpenCheckForm()
        {
            ListSelect form = new ListSelect();
            DataTable dt = OpenedStats.CheckStats();
            form.Text = "Checking";
            form.gridControl1.DataSource = dt;
            form.gridView1.PopulateColumns(dt);
            form.gridView1.Columns[0].Width = 140;
            form.gridView1.Columns[1].Width = 500;
            DevExpress.XtraEditors.Repository.RepositoryItemMemoEdit me = 
                new DevExpress.XtraEditors.Repository.RepositoryItemMemoEdit();
            me.WordWrap = true;
            form.gridView1.OptionsView.RowAutoHeight = true;
            form.gridView1.Columns[1].ColumnEdit = me;
            form.ShowDialog();
        }

        private void btCompile_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            if (OpenedStats.CheckStats().Rows.Count > 0)
                OpenCheckForm();
            string next_name = OpenedStats.wzfileinfo.GenerateNextVersionName();
            compilemap form = new compilemap();
            form.tMapName.Text = OpenedStats.wzfileinfo.original_mapname;
            if(next_name == "")
                form.tMapNameWithVersion.Text = OpenedStats.wzfileinfo.original_mapname;
            else
                form.tMapNameWithVersion.Text = next_name;
            DialogResult dr = form.ShowDialog();
            if (dr == System.Windows.Forms.DialogResult.OK)
            {
                if (form.tMapNameWithVersion.Text.Trim() == "")
                    MessageBox.Show("You entered empty map name. Operation canceled.");
                else
                {
                    if (form.tMapNameWithVersion.Text.Trim().ToLower()
                                        == OpenedStats.wzfileinfo.original_mapname.Trim().ToLower())
                    {
                        MessageBox.Show("Operation cancelled! you entered map name equal to old name of map. Change map name before compile it!");
                    }
                    else
                    {
                        wzSaveDialog.FileName = OpenedStats.wzfileinfo.map_prefix + form.tMapNameWithVersion.Text;
                        if (wzSaveDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                        {
                            OpenedStats.CompileMap(form.tMapNameWithVersion.Text, wzSaveDialog.FileName, OpenedFile);
                        }
                    }
                }
            }
            else
            {
            }
        }

        private void linkLoadMod_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            string wzpath = Properties.Settings.Default.WZdir;
            if (!CheckWzDir(wzpath))
                return;
            if (OpenedFile == "")
            {
                MessageBox.Show("Sorry! First you need open file.");
                return;
            }
            if (!File.Exists(OpenedFile))
            {
                MessageBox.Show("Error: Opened file not exists! WTF?? close program and try reopen.");
                return;
            }
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {

                if (MessageBox.Show(
                    "All current data will be overriden. You need save current file. Confirm?", "Warning", MessageBoxButtons.OKCancel)
                    == System.Windows.Forms.DialogResult.OK)
                {
                    SaveCurFile();
                    OpenedStats.LoadMod(openFileDialog1.FileName, OpenedFile);
                }
            }
        }

        private void simpleButton1_Click_1(object sender, EventArgs e)
        {
        }

        private void linkSensor_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("sensor.txt");
        }

        private void linkBrain_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("brain.txt");
        }

        private void menuCheck_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
        {
            OpenCheckForm();
        }

        private void linkTree1_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            //if (!OpenedStats.FileList.ContainsKey("research.txt"))
            //    return;
            //if (!OpenedStats.FileList.ContainsKey("prresearch.txt"))
            //    return;
            //ResTreeForm form = new ResTreeForm();
            //DataTable dt_res = OpenedStats.FileList["research.txt"].data;
            //DataTable dt_join = OpenedStats.FileList["prresearch.txt"].data.Copy();
            //for (int i = 0; i < dt_res.Rows.Count; i++)
            //{
            //    dt_join.Columns.Add(dt_res.Columns[i]);
            //}
            //for(int i=0; i<dt_join.Rows.Count; i++)
            //{
            //    DataRow[] res_row = dt_res.Select("Code = " + dt_join.Rows[i]["PreresearchCode"].ToString());
            //    if (res_row.Length < 1)
            //        continue;
            //    foreach (DataColumn col in dt_res.Columns)
            //    {
            //        dt_join.Rows[i][col.ColumnName] = res_row[0][col];
            //    }
            //}
            //form.treeList1.DataSource = dt_res;
            //form.treeList1.KeyFieldName = "";
            //form.treeList1.ParentFieldName = "";
        }

        private void linkFeatures_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("features.txt");
        }

        private void simpleButton1_Click(object sender, EventArgs e)
        {
            MessageBox.Show(OpenedStats.calls.ToString());
        }

        public bool loading_panel_opened = false;
        public void ShowLoadingPanel(string message = "")
        {
            if (!loading_panel_opened)
            {
                SplashScreenManager.ShowForm(typeof(WaitForm1));
                loading_panel_opened = true;
            }
        }

        public void HideLoadingPanel(string message = "")
        {
            if (loading_panel_opened)
            {
                SplashScreenManager.CloseForm();
                loading_panel_opened = false;
            }
        }

        private void linkBodyPropulsion_LinkClicked(object sender, DevExpress.XtraNavBar.NavBarLinkEventArgs e)
        {
            OpenStatEditor_forCommonFile("bodypropulsionimd.txt");
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            //Environment.Exit();
        }








    }
}