﻿/*
	This file is part of Warzone Stat Editor by crab
    Copyright © 2011, 2012 Vladimir Salnikov
 
	Warzone Stat Editor by crab is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	Warzone Stat Editor by crab is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Warzone Stat Editor by crab; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Ionic.Zip;
using System.Data;
using System.Text.RegularExpressions;
using System.Timers;
using System.ComponentModel;
using System.Threading;

namespace WZStats
{
    public interface VirtualForm
    {
        void ShowLoadingPanel(string message = "");
        void HideLoadingPanel(string message = "");
    }

    public class Stats
    {
        public class StatFile
        {
            public string path;
            public bool Loaded; 
            public bool HaveHeader;
            public DataTable data;
            public bool IsNames;
            public Dictionary<string, string> names_data;
            public string NamesFile;
            public bool NeedSaveInAnyCase;
            public int[] KeyColumns;
            public int Checking_MaxLines;
            public string FileDescription;
            public bool GhostFileFromOriginalWz;
            public string stored_first_line;
        }
        public partial class Linker
        {
            public string link_file = ""; 
            public string[] lfiles = null;
            public string rkey = "";
            public string link_rkey = "";
            public string link_lkey = "";
            public string lkey = "";
            public string[] linkcolumns = null; //delim by |
            public string CountColumnName = "";
            public string linkTypeFieldName = "";
            public string[] linkTypes = null;  //delim by |
            public Linker(string source, string CountColumnName)
            {
                string[] format_fields = source.Split(';');
                foreach (string format_field in format_fields)
                {
                    string[] cells = format_field.Trim().Split('=');
                    if (cells.Length != 2)
                    {
                        Exception err = new Exception("Invalid format of value_list (cells.Lenght != 2). value_list=" + source);
                        throw err;
                    }
                    cells[0] = cells[0].Trim();
                    cells[1] = cells[1].Trim();
                    switch (cells[0])
                    {
                        case "link_file":
                            link_file = cells[1].Trim(); break;
                        case "lfile":
                            lfiles = cells[1].Split('|'); break;
                        case "rkey":
                            rkey = cells[1]; break;
                        case "link_rkey":
                            link_rkey = cells[1]; break;
                        case "link_lkey":
                            link_lkey = cells[1]; break;
                        case "lkey":
                            lkey = cells[1]; break;
                        case "linkcolumns":
                            linkcolumns = cells[1].Split('|'); break;
                        case "linkTypeFieldName":
                            linkTypeFieldName = cells[1]; break;
                        case "linkTypes": 
                            linkTypes = cells[1].Split('|'); break;
                        default:
                            Exception err = new Exception("Invalid field name '" + cells[0] + "' in value_list. value_list=" + source);
                            throw err;
                    }
                }
                this.CountColumnName = CountColumnName;
            }
            public bool CheckLink(Stats stats, string RFile, out string ErrMsg)
            {
                ErrMsg = "";
                if(lfiles == null)
                    ErrMsg += "\r\nL-files was not found";
                //check right file
                List<string> columns_found = new List<string>(20);
                if (!stats.FileList.ContainsKey(RFile))
                    ErrMsg += "\r\nRFile was not found in filelist, filename=" + RFile;
                else
                {
                    if (!stats.FileList[RFile].Loaded)
                        ErrMsg += "\r\nRFile was not loaded, filename=" + RFile;
                    else
                    {
                        foreach(string col in this.linkcolumns)
                            if (!columns_found.Contains(col))
                                if (stats.FileList[RFile].data.Columns.Contains(col))
                                    columns_found.Add(col);
                        if(!stats.FileList[RFile].data.Columns.Contains(this.rkey))
                            ErrMsg += "\r\nRFile doesn't contain column rkey=" + rkey + ", filename=" + RFile;
                    }
                }

                //check left file(s)
                if (lfiles != null)
                    foreach (string left_file in this.lfiles)
                    {
                        if (!stats.FileList.ContainsKey(left_file))
                            ErrMsg += "\r\nLFile was not found in filelist, filename=" + left_file;
                        else
                        {
                            if (!stats.FileList[left_file].Loaded)
                                ; //ErrMsg += "\r\nLFile was not loaded, filename=" + left_file;
                            else
                            {
                                foreach (string col in this.linkcolumns)
                                    if (!columns_found.Contains(col))
                                        if (stats.FileList[left_file].data.Columns.Contains(col))
                                            columns_found.Add(col);
                                if (!stats.FileList[left_file].data.Columns.Contains(this.lkey))
                                    ErrMsg += "\r\nLFile doesn't contain column lkey=" + this.lkey + ", filename=" + left_file;
                            }
                        }
                    }
                //check link file
                if (!stats.FileList.ContainsKey(this.link_file))
                    ErrMsg += "\r\nlink_file was not found in filelist, filename=" + this.link_file;
                else
                {
                    if (!stats.FileList[this.link_file].Loaded)
                        ErrMsg += "\r\nlink_file was not loaded, filename=" + this.link_file;
                    else
                    {
                        foreach (string col in this.linkcolumns)
                            if (!columns_found.Contains(col))
                                if (stats.FileList[this.link_file].data.Columns.Contains(col))
                                    columns_found.Add(col);
                        if (!stats.FileList[this.link_file].data.Columns.Contains(this.link_rkey))
                            ErrMsg += "\r\nLlink_file doesn't contain column link_rkey=" + this.link_rkey + ", filename=" + this.link_file;
                        if (!stats.FileList[this.link_file].data.Columns.Contains(this.link_lkey))
                            ErrMsg += "\r\nLlink_file doesn't contain column link_lkey=" + this.link_lkey + ", filename=" + this.link_file;
                    }
                }


                //if (columns_found.Count != this.linkcolumns.Length)
                //    ErrMsg += "\r\nLNot all columns was found. Columns list: " + this.linkcolumns;

                return ErrMsg == "";
            }
            public void Join(Stats stats, string destination_filename, ref DataTable destination, ref DataTable source)
            {
                DataTable link_dt = destination;
                DataTable L_dt = source;
                //bool NeedJoin = false;
                List<string> NativeColumns = new List<string>();
                foreach (DataColumn dc in L_dt.Columns)
                {
                    if (!link_dt.Columns.Contains(dc.ColumnName))
                    {
                       // NeedJoin = true;
                        link_dt.Columns.Add(dc.ColumnName, dc.DataType);
                    }

                }
                //if (NeedJoin)
                DataRow[] fields = stats.statfields.SelectFields(destination_filename);
                
                    foreach (DataRow dr in link_dt.Rows)
                    {
                        DataRow[] joinrows = L_dt.Select(this.lkey + "= '" + dr[this.link_lkey] + "'");
                        if (joinrows.Length > 0)
                            foreach (DataColumn dc in L_dt.Columns)
                            {
                                bool NativeField = false; 
                                foreach (DataRow field in fields)
                                {
                                    if (field["Caption"].ToString() == dc.ColumnName)
                                    {
                                        NativeField = true;
                                        break;
                                    }
                                }
                                if(!NativeField)
                                    dr[dc.ColumnName] = joinrows[0][dc];
                            }
                    }
            }
        }
        public partial class ListFromFile
        {
            public string lfile;
            public string lKeyColumn;
            public string lNameColumn;
            public ListFromFile(string expression)
            {
                string[] format_fields = expression.Split(';');
                foreach (string format_field in format_fields)
                {
                    string[] cells = format_field.Trim().Split('=');
                    if (cells.Length != 2)
                    {
                        Exception err = new Exception("StatsMetadata error: Invalid format of expression (cells.Lenght != 2). expression=" + expression);
                        throw err;
                    }
                    cells[0] = cells[0].Trim();
                    cells[1] = cells[1].Trim();
                    switch (cells[0])
                    {
                        case "lfile":
                            lfile = cells[1].Trim(); break;
                        case "lKeyColumn":
                            lKeyColumn = cells[1].Trim(); break;
                        case "lNameColumn":
                            lNameColumn = cells[1].Trim(); break;
                        default:
                            Exception err = new Exception("StatsMetadata error: Invalid field name '" + 
                                                    cells[0] + "' in value_list. value_list=" + expression);
                            throw err;
                    }
                }
            }
        }

        public partial class WZFileInfo
        {
            private Stats stats;
            public string current_mapname;
            public string original_mapname;
            public string clear_filename;
            public string map_prefix;
            public WZFileInfo(Stats openedstats)
            {
                stats = openedstats;
                current_mapname = original_mapname = clear_filename = map_prefix = "";
            }
            public void UpdateFileInfoRecord(string lev_file_name, string lev_file_content)
            {
                current_mapname = ExtractMapName(lev_file_name, lev_file_content);
                if (!stats.FileList["wzfileinfo.txt"].Loaded)
                { // make new map name
                    StatFile st = stats.FileList["wzfileinfo.txt"];
                    st.data = new DataTable("wzfileinfo.txt");
                    DataRow[] fields = stats.statfields.SelectFields("wzfileinfo.txt");
                    stats.SetDataColumns(ref st.data, fields);
                    DataRow r = st.data.NewRow();
                    r[0] = "original map name"; 
                    r[1] = current_mapname;
                    original_mapname = r[1].ToString();
                    st.data.Rows.Add(r);
                    stats.FileList["wzfileinfo.txt"].Loaded = true;
                }
                else
                { // fill map name from .lev-file and original name from history-file
                    foreach(DataRow row in stats.FileList["wzfileinfo.txt"].data.Rows)
                    {
                        string key = row[0].ToString();
                        string value = row[1].ToString();
                        if (key == "original map name")
                        {
                            original_mapname = value;
                        }
                    }
                    if (original_mapname == "")
                        original_mapname = current_mapname;
                }
                
                map_prefix = ExtractMapPrefix(lev_file_name);
            }

            string mychars = "0123456789ABCDEFGHIJKLMNOPQRSTUWXYZ";
            public string GenUniqNum(string c)
            {
                int mi = mychars.Length - 1;
                int c0 = mychars.IndexOf(c[0]);
                int c1 = mychars.IndexOf(c[1]);
                if (c0 > mi || c0<0 || c.Length!=2 || c1>mi)
                    return "01";
                else
                {
                    if (c1<0)
                    {
                        return c[0] + "1";
                    }
                    else if (c1 == mi)
                    {
                        if(c0 == mi)
                            return "01"; //switch from ZZ to 01
                        else
                            return mychars[c0 + 1] + "0";
                    }
                    else
                    {
                        return c[0] + "" + mychars[c1 + 1];
                    }
                }
            }

            public string MakeNameAuto(string current_name)
            {

                if (current_name == original_mapname)
                    return original_mapname + "_01";
                else
                {
                    if(Regex.IsMatch(current_name,".+[_]..$") && current_name.Length>4)
                    {
                        return current_name.Remove(current_name.Length-2) + 
                                            GenUniqNum(current_name.Substring(current_name.Length-2,2));

                    }else
                    {
                        return current_name + "_01";
                    }
                }
            }

            private string ExtractMapPrefix(string level_filename)
            {
                return level_filename.Substring(0, 3);
            }

            public string ExtractMapName(string level_filename, string level_file_contents)
            {

                Dictionary<string, string> data;
                stats.ParseNamesFile(out data, level_file_contents);
                if (!data.ContainsKey("level"))
                {
                    Exception err = new Exception("ExtractMapName: level key not found in file: " + level_filename);
                    throw err;
                }
                string level = data["level"];
                level = Regex.Replace(level, @"^(S|s)(K|k)[-]", "");
                level = Regex.Replace(level, @"[-]([t]|[T])([1]|[2]|[3]|[4]|[5]|[6]|[7])$", "");
                return level;
            }
            public void UpdateMapName(string new_name, ref string level_filename, ref string level_file_contents)
            {
                string old_name = ExtractMapName(level_filename,level_file_contents);
                string[] lines = level_file_contents.Split('\n');
                for (int i = 0; i < lines.Length; i++)
                {
                    if(Regex.IsMatch(lines[i],@"level[\s].+"))
                    {
                        lines[i] = lines[i].Replace(old_name, new_name);
                    }
                }
                level_filename = level_filename.Replace(old_name, new_name);
                level_file_contents = String.Join("\n", lines);
            }

            public string GenerateNextVersionName()
            {
                //1. is not a map - original map name is empty
                if (original_mapname == "")
                    return "";
                //2. is recently modded map - original name filled, mapname empty or equal to original name
                if (current_mapname == original_mapname || current_mapname == "")
                {
                    return MakeNameAuto(original_mapname);
                }
                //3. is saved map version both names filled and not same
                if (current_mapname != original_mapname)
                {
                    return MakeNameAuto(current_mapname);
                }
                //
                // if(mapname
                return "";
            }
        }

        public partial class CodeFunction
        {/*class which provides automatical calculations of fields with type "code functions"
          * Perform lazy evaluation of code functions
          * Warning: this class may cause strange errors          
          */
            private Stats stats;
            private Dictionary<string, List<string>> ResultingFields; //key - filename, value - list of fields
            private Dictionary<string, Dictionary<string, bool>> DependentFields; //key - filename, value - list of fields
            private Dictionary<string, int> DependentFiles; //key - filename
            public delegate object CalculateFuncRef_Method(DataRow row);
            public CalculateFuncRef_Method CalculateFuncRef;
            public DateTime LastUpdateTry; //to prevent update data while dependetn table is updating in loop
            public bool UpdateTimerStarted;
            public Dictionary<string, bool> AttachedEvents; //key - filename, value - true if events attached
            public SynchronizationContext context;
            public CodeFunction(Stats stats, CalculateFuncRef_Method CalculateFieldMethod)
            {//CalculateFieldMethod - method, which calculates value of code function
                context = SynchronizationContext.Current ?? new SynchronizationContext();
                this.stats = stats;
                DependentFiles = new Dictionary<string, int>();
                ResultingFields = new Dictionary<string, List<string>>();
                DependentFields = new Dictionary<string, Dictionary<string, bool>>();
                CalculateFuncRef = CalculateFieldMethod;
                UpdaterTimer = new System.Timers.Timer();
                UpdaterTimer.Elapsed += ElapsedEvent_RunUpdaterByTimer;
                //SynchronizingObject  - timer event and update of data should be performed in synch with main form (main thread)
                UpdaterTimer.SynchronizingObject = stats.SynchronizingObjectForm as System.ComponentModel.ISynchronizeInvoke;
                UpdateTimerStarted = false;
                LastUpdateTry = DateTime.MinValue;
                AttachedEvents = new Dictionary<string, bool>();
                if (!stats._codeFunctions.Contains(this))
                    stats._codeFunctions.Add(this);
            }


            public void AddDestinationField(string filename, string fieldname)
            {
                if(!ResultingFields.ContainsKey(filename))
                    ResultingFields.Add(filename,new List<string>());
                if(!ResultingFields[filename].Contains(fieldname))
                    ResultingFields[filename].Add(fieldname);
            }

            public void AddDependentField(string filename, string fieldname)
            {
                if (!DependentFields.ContainsKey(filename))
                    DependentFields.Add(filename, new Dictionary<string, bool>());
                if (!DependentFields[filename].ContainsKey(fieldname))
                    DependentFields[filename].Add(fieldname, true);
            }

            public void AddDependetnFile(string filename)
            {
                if (!DependentFiles.ContainsKey(filename))
                    DependentFiles.Add(filename, 0);
            }
            public void AddDependentFiles(string[] filenames)
            {
                foreach (string filename in filenames)
                    if (!DependentFiles.ContainsKey(filename))
                        DependentFiles.Add(filename, 0);
            }

            private void CalculateInThread()
            {
                //private Dictionary<string, Dictionary<int, object>[]> ThUpdateList; //file - data, row - fields, field index - value
                Dictionary<string, Dictionary<int, object>[]> ThUpdateList;
                ThUpdateList = new Dictionary<string, Dictionary<int, object>[]>();
                foreach (string res_file in ResultingFields.Keys)
                {
                    List<string> res_fields = ResultingFields[res_file];
                    if (stats.FileList[res_file].Loaded)
                    {
                        if (!ThUpdateList.ContainsKey(res_file))
                        {
                            DataTable dt = stats.FileList[res_file].data;
                            Dictionary<int, object>[] row_values = new Dictionary<int, object>[dt.Rows.Count];
                            for (int i = 0; i < dt.Rows.Count; i++)
                            {
                                row_values[i] = new Dictionary<int, object>();
                                object function_result = CalculateFuncRef(dt.Rows[i]);
                                for (int f = 0; f < res_fields.Count; f++)
                                    row_values[i].Add(f, function_result);
                            }
                            ThUpdateList.Add(res_file, row_values);
                        }
                    }
                }
                try
                {
                    context.Send(s =>
                    {
                        VirtualForm form = stats.SynchronizingObjectForm as VirtualForm;
                        form.ShowLoadingPanel();
                        foreach (string res_file in ResultingFields.Keys)
                        {
                            List<string> res_fields = ResultingFields[res_file];
                            if (stats.FileList[res_file].Loaded)
                            {
                                bool need_switch_off_events = DependentFiles.ContainsKey(res_file) || DependentFields.ContainsKey(res_file);
                                DataTable dt = stats.FileList[res_file].data;

                                //1. if function called from one of dependent files - then we need prevent infinity recursion
                                if (need_switch_off_events)
                                    DetachEventsFromFile(res_file);

                                //2. Update calculated values in Destination table
                                for (int i = 0; i < dt.Rows.Count; i++)
                                {
                                    for (int f = 0; f < res_fields.Count; f++)
                                    {
                                        if (ThUpdateList[res_file][i][f] == null)
                                            dt.Rows[i][res_fields[f]] = -1;
                                        else
                                            dt.Rows[i][res_fields[f]] = ThUpdateList[res_file][i][f];
                                    }
                                }

                                //3. if we switched off events due to prevent recursion then we need switch it on
                                if (need_switch_off_events)
                                    AttachEventsToFile(res_file);
                            }
                        }
                        form.HideLoadingPanel();
                    }, null);
                }
                catch (InvalidAsynchronousStateException exp)
                {
                    //catch synch expection to hide error messages to user
                    //this error occures if user closed program before we end
                }
            }
            
            public void UpdateCalculationResults()
            {
                context = SynchronizationContext.Current ?? new SynchronizationContext();
                System.Threading.Thread th = new System.Threading.Thread(CalculateInThread);
                th.Start();
            }
            private void Column_Changed(object sender, DataColumnChangeEventArgs e, string filename)
            {
                if (!DependentFields.ContainsKey(filename))
                    return;
                Dictionary<string, bool> fields = DependentFields[filename];
                if (!fields.ContainsKey(e.Column.ColumnName))
                    return;
                bool ActuallyChanged = false;
                if (e.Row.RowState == DataRowState.Modified)
                {
                    if (e.Row[e.Column, DataRowVersion.Original] == null)
                        ActuallyChanged = true;
                    else
                    {
                        if (e.Column.DataType == typeof(Int32))
                            ActuallyChanged = (int)e.Row[e.Column] != (int)e.Row[e.Column, DataRowVersion.Original];
                        else if (e.Column.DataType == typeof(String))
                            ActuallyChanged = (string)e.Row[e.Column] != (string)e.Row[e.Column, DataRowVersion.Original];
                        else
                            ActuallyChanged = e.Row[e.Column] != e.Row[e.Column, DataRowVersion.Original];
                    }
                }
                else
                    ActuallyChanged = true;
                if (ActuallyChanged)
                    PerformLazyUpdate();
            }
            private void ElapsedEvent_RunUpdaterByTimer(object source, ElapsedEventArgs e)
            {
                if ((DateTime.Now - LastUpdateTry).TotalSeconds < 4)
                    return;
                if (!UpdateTimerStarted)
                    return;
                LastUpdateTry = DateTime.Now;
                UpdaterTimer.Stop();
                UpdateCalculationResults();
                UpdateTimerStarted = false;
            }

            private void PerformLazyUpdate()
            {
                LastUpdateTry = DateTime.Now;
                if (!UpdateTimerStarted)
                {
                        UpdateTimerStarted = true;
                        UpdaterTimer.Interval = 5000;
                        UpdaterTimer.Start();
                }
            }

            public void PerformForcedUpdate()
            {
                UpdateTimerStarted = false; //cancel lazy update in case it was started before
                CalculateInThread(); 
            }

            public System.Timers.Timer UpdaterTimer;

            private void Row_Changed(object sender, DataRowChangeEventArgs e)
            {
                PerformLazyUpdate();
            }

            private void DetachEventsFromFile(string filename)
            {
                if (!AttachedEvents.ContainsKey(filename))
                    return;
                if (DependentFiles.ContainsKey(filename))
                {
                    stats.FileList[filename].data.RowChanged -= Row_Changed;
                    AttachedEvents.Remove(filename);
                    return;
                }
                if (DependentFields.ContainsKey(filename))
                {
                    foreach (string field in DependentFields[filename].Keys)
                    {
                        DataTable dt = stats.FileList[filename].data;
                        if (dt.Columns.Contains(field))
                        {
                            dt.ColumnChanged -= delegate(object sender, DataColumnChangeEventArgs e)
                            { Column_Changed(sender, e, filename); };
                            AttachedEvents.Remove(filename);
                            return;
                        }
                    }
                }
            }

            private void AttachEventsToFile(string filename)
            {
                if (AttachedEvents.ContainsKey(filename))
                    return; // skip if events already attached
                if (!stats.FileList[filename].Loaded)
                    return;
                if (DependentFiles.ContainsKey(filename))
                {
                    stats.FileList[filename].data.RowChanged += Row_Changed;
                    AttachedEvents.Add(filename, true);
                    return; 
                }
                if (DependentFields.ContainsKey(filename))
                {
                    foreach (string field in DependentFields[filename].Keys)
                    {
                        DataTable dt = stats.FileList[filename].data;
                        if (dt.Columns.Contains(field))
                        {
                            dt.ColumnChanged += delegate(object sender, DataColumnChangeEventArgs e)
                            { Column_Changed(sender, e, filename); };
                            AttachedEvents.Add(filename, true);
                            return;
                        }
                    }
                }
            }

            public void Start_Automatic_Calculation(string _only_one_filename="")
            {
                if (_only_one_filename != "")
                    AttachEventsToFile(_only_one_filename);
                else
                {
                    foreach (string filename in DependentFiles.Keys)
                        AttachEventsToFile(filename);
                    foreach (string filename in DependentFields.Keys)
                        AttachEventsToFile(filename);
                }
                PerformLazyUpdate(); //to be sure what we update all values after start of calculations
            }
        }

        public  WZFileInfo wzfileinfo;
        public bool NeedUpdateNames = false;
        public Dictionary<string, StatFile> FileList;
        public StatMetadata statfields;
        CodeFunction CodeFunction_ResearchPoints;
        public object SynchronizingObjectForm;
        private List<CodeFunction> _codeFunctions;

        public Stats(string FilesListcsv_filename, string StatsMetadata_filename, object SynchronizingObjectForm)
        {
            this.SynchronizingObjectForm = SynchronizingObjectForm;
            this._codeFunctions = new List<CodeFunction>();
            FileList = new Dictionary<string, StatFile>(30);
            string[] lines = File.ReadAllLines(FilesListcsv_filename);
            int rownumber = 1;
            foreach (string line in lines)
            {
                string[] cells = line.Split(',');
                if (cells.Length != 7)
                {
                    Exception e = new Exception(@"Loading list of files error: Invalid format of csv-file " + FilesListcsv_filename +
                                " File should have 4 columns. Row number: " + rownumber.ToString());
                    throw e;
                }
                if(FileList.ContainsKey(cells[0]))
                {
                    Exception e = new Exception(@"Loading list of files error: Filename used more than 1 time " + FilesListcsv_filename +
                                " Both columns in file should have unique values. Row number: " + rownumber.ToString());
                    throw e;
                }
                StatFile fileinfo = new StatFile();
                fileinfo.Loaded = false;
                fileinfo.path = cells[1];
                fileinfo.HaveHeader = cells[2].ToLower().Contains("header");
                fileinfo.IsNames = cells[2].ToLower().Contains("names");
                fileinfo.NamesFile = cells[3];
                fileinfo.NeedSaveInAnyCase = false;
                fileinfo.names_data = null;
                fileinfo.data = null;
                fileinfo.stored_first_line = "";
                string[] keys = cells[4].Split('|');

                string[] CheckParams = cells[5].Split(';');
                fileinfo.Checking_MaxLines = Int32.MaxValue;
                foreach (string checkparam in CheckParams)
                {
                    string[] pcells = checkparam.Split('=');
                    if (pcells.Length != 2)
                        continue;
                    string pkey = pcells[0].Trim();
                    string pvalue = pcells[1].Trim();
                    if (pkey == "MaxLines")
                    {
                        int val;
                        if(Int32.TryParse(pvalue, out val))
                            fileinfo.Checking_MaxLines = val;
                    }
                }
                fileinfo.FileDescription = cells[6];
                fileinfo.KeyColumns = new int[keys.Length];
                for(int i=0; i<keys.Length; i++)
                {
                    int keynum;
                    if(!Int32.TryParse(keys[i], out keynum))
                        keynum = 0;
                    fileinfo.KeyColumns[i] = keynum;
                }
                {
                    
                }
                if (FileList.ContainsValue(fileinfo))
                {
                    Exception e = new Exception(@"Loading list of files error: Path of file used more than 1 time " + FilesListcsv_filename +
                                " Both columns in file should have unique values. Row number: " + rownumber.ToString());
                    throw e;
                }
                FileList.Add(cells[0], fileinfo);
                rownumber++;
            }
            statfields = new StatMetadata();
            statfields.Load("stats_metadata.csv");
            wzfileinfo = new WZFileInfo(this);
            CodeFunction_ResearchPoints = new CodeFunction(this, GetResearchPoints);
            string[] tmp1 = new string[] { "research.txt", "prresearch.txt", "resultstructure.txt", "resultcomponent.txt" };
            foreach (string tmp_filename in tmp1)
            {
                DataRow[] rfs = statfields.SelectFields(tmp_filename);
                foreach (DataRow r in rfs)
                {
                    StatMetadata.ExType xt = (StatMetadata.ExType)r["ExType"];
                    if (xt != StatMetadata.ExType.CodeFunction)
                        CodeFunction_ResearchPoints.AddDependentField(tmp_filename, (string)r["Caption"]);
                }
            }
        }
        public void CheckZipFile(string WZzip_filepath)
        {
            if (!ZipFile.IsZipFile(WZzip_filepath, true))
            {
                Exception e = new Exception(@"Error loading data: Invalid zip-archive. Filename = " + WZzip_filepath);
                throw e;
            }

            //else if (!ZipFile.CheckZip(WZzip_filepath))
            //{
            //    //Exception e = new Exception(@"Error loading data: Invalid zip-archive.  Filename = " + WZzip_filepath);
            //    //throw e;
            //}
        }

        public void LoadMod(string mod_filename, string current_file)
        {
            Load(mod_filename, "", true);
            foreach (string key in FileList.Keys)
                FileList[key].NeedSaveInAnyCase = true;
            Save(current_file);
            ZipFile modzip = ZipFile.Read(mod_filename);
            ZipFile curzip = ZipFile.Read(current_file);
            
            foreach (ZipEntry entry in modzip)
            {
                if (entry.IsDirectory)
                    continue;
                string filename_short = Path.GetFileName(entry.FileName);
                //skip files which sould be loaded in call Load function above
                if (FileList.ContainsKey(filename_short))
                    continue;
                //skip map files
                if(Regex.IsMatch(entry.FileName,@"^multiplay.maps.+$"))
                    continue;
                if (Regex.IsMatch(entry.FileName, @".+\.lev$"))
                    continue;
                Ionic.Crc.CrcCalculatorStream entry_read_stream = entry.OpenReader();
                StreamReader str_reader = new StreamReader(entry_read_stream);
                string file_content = str_reader.ReadToEnd();
                curzip.UpdateEntry(entry.FileName, file_content);
            }
            curzip.Save();
            modzip.Dispose();
            curzip.Dispose();
        }

        public bool TryLoadTextFile(string txt_filename)
        {
            string file_content = File.ReadAllText(txt_filename);
            foreach (string fname in FileList.Keys)
            {
                StatFile st = FileList[fname];
                try
                {
                    if(!FileList[fname].IsNames)
                    {
                        st.data = new DataTable(fname);
                        if (LoadStats(fname, ref st.data, file_content, st.HaveHeader))
                        {
                            st.Loaded = true;
                            return true;
                        }
                    }
                }
                catch
                {
                    if(!st.Loaded)
                        st.data = null;
                }

            }
            return false;
        }

        private void LinkerEngine_StatDataTable_RKeyChanging(object sender, DataColumnChangeEventArgs e, string statfile, Linker linker)
        {
            if (e.Column.ColumnName != linker.rkey)
                return;       
            //unique code here
            DataTable r_dt = sender as DataTable;
            DataRow[] rows = r_dt.Select("[" + linker.rkey + "] = '" + e.ProposedValue + "'");
            if (rows.Length > 0)
            {
                Exception ex = new Exception("Error: duplicate key");
                throw ex;
            }
            //catch moment when user changing code of element and update link tables
            DataTable link_dt = FileList[linker.link_file].data;
            DataRow[] link_rows = link_dt.Select("[" + linker.link_rkey + "] = '" + e.Row[linker.rkey].ToString() + "'");
            if (link_rows.Length > 0)
            {
                foreach (DataRow row in link_rows)
                {
                    row[linker.link_rkey] = e.ProposedValue;
                }
            }
        }

        private void StatDataTable_RowDeleting(object sender, DataRowChangeEventArgs e, string statfile)
        {
            //we should delete associated rows in linked table
            int capt_col_index = statfields.data.Columns.IndexOf("Caption");
            int colExType = statfields.data.Columns.IndexOf("ExType");
            DataRow[] fields = statfields.SelectFields(statfile);
            foreach (DataRow field in fields)
            {
                if ((StatMetadata.ExType)field[colExType] == StatMetadata.ExType.LinkTable)
                {
                    string FieldExpr = field["Expression"].ToString();
                    if (FieldExpr != "")
                    {
                        Stats.Linker linker = new Stats.Linker(FieldExpr, field["Caption"].ToString());
                        string link_errors = "";
                        if (linker.CheckLink(this, statfile, out link_errors))
                        {
                            DataTable link_dt = FileList[linker.link_file].data;
                            List<DataRow> rows_to_del = new List<DataRow>();
                            foreach (DataRow link_row in link_dt.Rows)
                            {
                                if (link_row[linker.link_rkey].ToString() == e.Row[linker.rkey].ToString())
                                    rows_to_del.Add(link_row);
                            }
                            foreach (DataRow r_del in rows_to_del)
                                link_dt.Rows.Remove(r_del);
                        };
                    }
                } //end if type == LinkTable
            }
        }

        private void AttachTriggersToDT(DataTable dt_stat, string statfile)
        {
            dt_stat.RowDeleting += delegate(object sender, DataRowChangeEventArgs e)
            { StatDataTable_RowDeleting(sender, e, statfile); };

            //linker engine: handle situation when user changes key in right table
            int capt_col_index = statfields.data.Columns.IndexOf("Caption");
            int colExType = statfields.data.Columns.IndexOf("ExType");
            DataRow[] fields = statfields.SelectFields(statfile);
            foreach (DataRow field in fields)
            {
                if ((StatMetadata.ExType)field[colExType] == StatMetadata.ExType.LinkTable)
                {
                    string FieldExpr = field["Expression"].ToString();
                    if (FieldExpr != "")
                    {
                        Stats.Linker linker = new Stats.Linker(FieldExpr, field["Caption"].ToString());
                        string link_errors = "";
                        if (linker.CheckLink(this, statfile, out link_errors))
                        {
                            dt_stat.ColumnChanging += delegate(object sender, DataColumnChangeEventArgs e)
                            { LinkerEngine_StatDataTable_RKeyChanging(sender, e, statfile, linker); };
                        }
                    }
                }
            }
        }

        public bool IsFileLoaded(string statfile)
        {
            if (FileList.ContainsKey(statfile))
                if (FileList[statfile].Loaded)
                    return true;
            return false;
        }

        public void Load(string WZzip_filepath, string OnlyOne_filename = "", bool IsModLoading = false)
        {
            /*
             * function for load data in Stats object
             * function load data and performs neccessary operations
             * 
             * WZzip_filepath - zip-archive containing warzone map or mod
             * OnlyOne_filename - if specified then load will work only for this file of loaded archive
             * IsModLoading - in case load mod, we should skip updating map name from loaded mod
             * */

            try
            {
                CheckZipFile(WZzip_filepath);
            }
            catch(Exception err)
            {
                string wz_ext = WZzip_filepath.Substring(WZzip_filepath.Length - 3, 3).ToLower();
                if (wz_ext == ".wz" && wz_ext == ".crab")
                {
                    throw err;
                }
                else if (TryLoadTextFile(WZzip_filepath))
                {
                    ;
                    return;
                }else
                    throw err;
            }
            
            ZipFile zip = ZipFile.Read(WZzip_filepath);
            
            foreach(ZipEntry entry in zip)
            {
                if(entry.IsDirectory)
                    continue;
                string filename_short = Path.GetFileName(entry.FileName);
                if (FileList.ContainsKey(filename_short))
                {
                    if (FileList[filename_short].path == entry.FileName &&
                        (OnlyOne_filename == "" || filename_short==OnlyOne_filename) )
                    {
                        Ionic.Crc.CrcCalculatorStream entry_read_stream = entry.OpenReader();
                        StreamReader str_reader = new StreamReader(entry_read_stream);
                        string file_content = str_reader.ReadToEnd();
                        StatFile st = FileList[filename_short];
                        st.data = new DataTable(filename_short);
                        try
                        {
                            if (st.IsNames)
                            {
                                if (st.names_data != null)
                                    st.NeedSaveInAnyCase = true;
                                ParseNamesFile(out st.names_data, file_content);
                                st.Loaded = true;
                                NeedUpdateNames = true;
                            }
                            else
                            {
                                st.Loaded = LoadStats(filename_short, ref st.data,
                                    file_content,
                                    st.HaveHeader);
                            }
                        }
                        catch (Exception ex)
                        {
                            Exception newex = new Exception(@"Error: loading stat file failed
                                              FileName=" + filename_short
                                            + "\r\nError message: " + ex.Message);
                            throw newex;
                        }
                        FileList[filename_short] = st;
                    }
                }
            }


            //variant1: if it is not mod then we update map name
            //variant2: if we loading mod then we not need update map name
            if (!IsModLoading) 
            {
                foreach (ZipEntry entry in zip)
                {
                    //process level file
                    if (entry.IsDirectory)
                        continue;
                    string filename_short = Path.GetFileName(entry.FileName);
                    if (Regex.IsMatch(filename_short, @".+\.lev$"))
                    {
                        Ionic.Crc.CrcCalculatorStream entry_read_stream = entry.OpenReader();
                        StreamReader str_reader = new StreamReader(entry_read_stream);
                        string file_content = str_reader.ReadToEnd();
                        wzfileinfo.UpdateFileInfoRecord(filename_short, file_content);
                    }
                }
                wzfileinfo.clear_filename = Path.GetFileNameWithoutExtension(WZzip_filepath);
            }

            zip.Dispose();

            //add/update Name column in all stat tables
            if (OnlyOne_filename == "")
            {
                UpdateNames();
                CodeFunction_ResearchPoints.Start_Automatic_Calculation();
            }
            else
            {
                UpdateNames("", OnlyOne_filename);
                CodeFunction_ResearchPoints.Start_Automatic_Calculation(OnlyOne_filename);
            }

            //attach triggers to data tables to handle links on row delete and handle change code of elemtns
            foreach (string statfile in FileList.Keys)
            {
                if(IsFileLoaded(statfile))
                    if(!FileList[statfile].IsNames)
                        AttachTriggersToDT(FileList[statfile].data, statfile);//triggers init code requires all data loaded
            }
        }

        //Update column containing item name
        private void UpdateNameColumn(string stat_filename)
        {
            /*
             * Function create/update name column in selected stat-file
             * param1 - name of selected stat file
             */
            if (FileList[stat_filename].NamesFile == "")
                return;
            DataTable dt = FileList[stat_filename].data;
            if (!dt.Columns.Contains("Code"))
                return;
            if(!dt.Columns.Contains("Name"))
                dt.Columns.Add("Name", typeof(String));
            foreach (DataRow row in dt.Rows)
            {
                string code = row["Code"].ToString();
                string name = "name not found";
                //search name in all names files
                foreach (string stfile in FileList.Keys) 
                {
                    if (FileList[stfile].IsNames && FileList[stfile].Loaded)
                        if (FileList[stfile].names_data.ContainsKey(code))
                        {
                            name = FileList[stfile].names_data[code];
                            break;
                        }
                }
                row["Name"] = name;
            }
        }

        public void UpdateNames(string OnlyOneNameFile = "", string OnlyOneStatFile = "")
        {
            /*
             * Function create/update name column in all stat-tables aand try fill this column from linked name-file
             * param1 - filter by name file
             * if param1 specified then we update only stat-tables which have linked name file equal to param1
             */
            foreach (string statfile in FileList.Keys)
            {
                if (OnlyOneStatFile != "")
                    if (statfile != OnlyOneStatFile)
                        continue;
                if (!FileList[statfile].Loaded)
                    continue;
                string namesfile = FileList[statfile].NamesFile;
                if (!FileList.ContainsKey(namesfile))
                    continue;
                if (OnlyOneNameFile != "")
                    if (OnlyOneNameFile != namesfile)
                        continue;
                if (!FileList[namesfile].IsNames)
                    continue;
                if (!FileList[namesfile].Loaded)
                    continue;
                UpdateNameColumn(statfile);
            }
            NeedUpdateNames = false;
        }

        public void ParseNamesFile(out Dictionary<string, string> names_data, string file)
        {
            names_data = new Dictionary<string,string>(2000);
            //deal with comments and etc.
            file = Regex.Replace(file, @"(/\*([^*]|[\r\n]|(\*+([^*/]|[\r\n])))*\*+/)|(//.*)", "");
            file = Regex.Replace(file, @"(([\r]|[\n]|[\r\n])+)|(^([\r]|[\n]|[\r\n]))", "\n");
            file = Regex.Replace(file, @"([\t]|[ ])+", " ");
            file = Regex.Replace(file, @"[\t]([\r]|[\n]|[\r\n])", "\n");
            file = file.Trim();
            file = file.Replace("_(\"","").Replace("\")","").Replace("\"","");
            string[] delim = new string[1];
            delim[0] = "\n";
            string[] lines = file.Split(delim, StringSplitOptions.RemoveEmptyEntries);
            foreach (string line in lines)
            {
                string tmp_line = line.Trim();
                if (tmp_line == "") continue;
                int ti = tmp_line.IndexOf(' ');
                if (ti < 0) continue;
                string key = tmp_line.Substring(0, ti);
                string name = "name loading error";
                if (tmp_line.Length >= ti + 1)
                    name = tmp_line.Substring(ti + 1);

                if(names_data.ContainsKey(key))
                {
                    if (names_data[key] == "")
                    {
                        names_data[key] = name;
                    }
                    //if key exists - then skip it
                }else
                {
                    names_data.Add(key, name);
                }
            }
        }

        protected void SetDataColumns(ref DataTable data, DataRow[] fields)
        { //Columns setup for datatable
            int colExType = statfields.data.Columns.IndexOf("ExType");
            int colIsInteger = statfields.data.Columns.IndexOf("IsInteger");
            for (int i = 0; i < fields.Length; i++)
            {
                DataColumn col = new DataColumn();
                col.ColumnName = fields[i]["Caption"].ToString();
                if ((bool)fields[i][colIsInteger] == true)
                {
                    col.DataType = typeof(Int32);
                }
                else
                {
                    col.DataType = typeof(String);
                }
                StatMetadata.ExType extype = (StatMetadata.ExType)fields[i][colExType];
                switch (extype)
                {
                    case StatMetadata.ExType.Expression:
                        col.Expression = fields[i]["Expression"].ToString().Replace(';', ',');
                        break;

                }
                data.Columns.Add(col);
            }
        }


        private bool TryLoadRow(string[] cells, DataRow row, DataRow[] fields, out string err_text)
        {  /*Function used to loading rows in LoadStats functions */
            err_text = "";
            for (int i = 0; i < cells.Length; i++)
            {
                string clear_cell_value = cells[i].Trim();
                if ((bool)fields[i]["IsInteger"])
                {
                    int r = 0;
                    if (Int32.TryParse(clear_cell_value, out r))
                        row[i] = r;
                    else
                    {
                        if (clear_cell_value != "")
                        {
                            err_text = @"Error loading data (LoadStats): Cannot convert value to type Int. Value="
                                            + cells[i] + " Line= '" + String.Join(",", cells) + "'";
                            return false;
                        }
                    }
                }
                else
                    row[i] = clear_cell_value;
            }
            return true;
        }

        public bool LoadStats(string filename, ref DataTable data, string filecontent, bool HeaderRow)
        {
            DataRow[] fields = statfields.SelectFields(filename);
            SetDataColumns(ref data, fields);
            string[] delim = new string[1];
            delim[0] = "\n";
            string[] lines = filecontent.Split(delim, StringSplitOptions.None);
            FileList[filename].stored_first_line = "";
            for (int ln = 0; ln < lines.Length - 1; ln++) //LAST LINE NOT READ (WZ RULE) usually last line is just emty string, but sometime there placec string which not read by WZ
            {
                string line = lines[ln];
                if (line.Trim() == "")
                    continue;

                if (HeaderRow)
                {
                    HeaderRow = false;
                    continue;
                }
                string[] cells = line.Split(',');
                if(cells.Length > fields.Length)
                {
                    if (fields.Length == 0)
                    {
                        return false;
                    }
                    string emg = String.Format("Error loading data (LoadStats): Invalid number of columns in csv-file. " +
                                                "\r\nNumber of columns should be lesser or equal to number of columns in field description file (stats_metadata.csv)" +
                                                "\r\nLine number = {1}, Expected columns count: {2},\r\nline={3}", filename, ln, fields.Length, line);
                    Exception e = new Exception(emg);
                    throw e;
                }
                DataRow row = data.NewRow();
                int colExType = statfields.data.Columns.IndexOf("ExType");
                int colIsInteger = statfields.data.Columns.IndexOf("IsInteger");
                int colCodeFunctionType = statfields.data.Columns.IndexOf("CodeFunctionType");

                ClassResearch clres = new ClassResearch(this);
                string err_text = "";

                if (ln == 0)
                {
                    if (!TryLoadRow(cells, row, fields, out err_text))//try skip header assuming it has errors in integer values
                    {
                        FileList[filename].stored_first_line = line;
                        //FileList[filename].HaveHeader = true; //remember what we skipped header, so in save function we should write header
                        //note: this code makes compatible 2.3.9 stats and master stats
                        continue;
                    }
                }
                if (!TryLoadRow(cells, row, fields, out err_text))
                {
                    Exception exp = new Exception(err_text);
                    throw exp;
                }

                //loop calculated columns (or columns which was not included into file due to error)
                for (int i = cells.Length; i < fields.Length; i++)
                {
                    //implemented code functions
                    if ((StatMetadata.ExType)fields[i][colExType] == StatMetadata.ExType.CodeFunction)
                    {
                        if ((StatMetadata.FuncType)fields[i][colCodeFunctionType] == StatMetadata.FuncType.ResearchPoints)
                        {
                            CodeFunction_ResearchPoints.AddDestinationField(filename, (string)fields[i]["Caption"]);
                        }
                    }
                }

                data.Rows.Add(row);
            }

            if (FileList[filename].KeyColumns != null)
                if (FileList[filename].KeyColumns.Length == 1)
                {
                    int key_index = FileList[filename].KeyColumns[0];
                    //just try make unqiue!!
                    //if fail - not bad
                    try
                    {
                        data.Columns[key_index].Unique = true;
                    }
                    catch (Exception err)
                    {
                        
                    }
                }

            return true;
        }
        public int calls = 0;

        private object GetResearchPoints(DataRow item_row)
        {
            calls++;
            if (!item_row.Table.Columns.Contains("Code"))
                return null;
            ClassResearch clres = new ClassResearch(this);
            int rp = clres.ComputeResearchPoints(item_row["Code"].ToString());
            if (rp > 0)
                return rp;
            else
                return null;
        }

        public void CompileMap(string new_mapname, string wz_filename, string crab_filename)
        {
            wzfileinfo.current_mapname = new_mapname;
            if (File.Exists(wz_filename) && wz_filename != crab_filename)
                    File.Delete(wz_filename);
            if (File.Exists(crab_filename) && wz_filename != crab_filename)
                File.Copy(crab_filename, wz_filename);
            wzfileinfo.current_mapname = new_mapname;
            UpdateStatFile(wz_filename);
        }

        public void Save(string wzfilename, string sourcewzfile="")
        {
            /* wzfilename - file name of file which we need to renew or create
             * sourcewzfile - 
             * Save function saves only stat-files from filelist.csv. all aother files not saved, 
             * so we need to copy from source then renew source archive
             * */
            if (sourcewzfile != "" && sourcewzfile != wzfilename)
            {
                File.Copy(sourcewzfile, wzfilename);
            }
            if(!Regex.IsMatch(wzfilename,@".+\.crab$"))
                wzfilename += ".crab";
            UpdateStatFile(wzfilename);
        }

        private void UpdateStatFile(string wzfilename, bool newfile=false)
        {
            /* update wz-file or create new wz-file */
            ZipFile zip;
            if(File.Exists(wzfilename))
                zip = ZipFile.Read(wzfilename);
            else
                zip = new ZipFile(wzfilename);

            //check map name and update if neccesary
            //MAP NAMES GET UNIQUE NEW NAME EVERY SAVE
            foreach (ZipEntry entry in zip)
            {
                if (entry.IsDirectory)
                    continue;
                //process level file
                string levfile = Path.GetFileName(entry.FileName);
                if (Regex.IsMatch(levfile, @".+\.lev$"))
                {
                    Ionic.Crc.CrcCalculatorStream entry_read_stream = entry.OpenReader();
                    StreamReader str_reader = new StreamReader(entry_read_stream);
                    string file_content = str_reader.ReadToEnd();
                    string new_lev_file = levfile;
                    wzfileinfo.UpdateMapName(wzfileinfo.current_mapname, ref new_lev_file, ref file_content);
                    if(new_lev_file != levfile)
                    {
                        zip.RemoveEntry(levfile);
                        zip.AddEntry(new_lev_file, file_content);
                        wzfileinfo.UpdateFileInfoRecord(new_lev_file, file_content);
                        break;
                    }
                    
                }
            }

            List<string> NamesFiles_ToSave = new List<string>(10);
            int capt_col_index = statfields.data.Columns.IndexOf("Caption");
            int colExType = statfields.data.Columns.IndexOf("ExType");
            int colIsInteger = statfields.data.Columns.IndexOf("IsInteger");
            foreach (string statfilename in FileList.Keys)
            {
                StatFile st = FileList[statfilename];
                string sel_fields = String.Format("filename = '{0}' AND ExpressionType<>'{1}'", 
                                        statfilename,StatMetadata.GetExType(StatMetadata.ExType.CodeFunction));
                DataRow[] fields = statfields.data.Select(sel_fields);
                if (!st.IsNames && st.Loaded)
                {
                    StringBuilder filecontent = new StringBuilder();
                    int colCode = -1;
                    int colName = -1;
                    StatFile st_namesfile = null;
                    string namesfilename = st.NamesFile;
                    bool SaveNamesToo = st.data.Columns.Contains("Code") 
                                            && st.data.Columns.Contains("Name") 
                                            && st.NamesFile != "";

                    if (SaveNamesToo)
                    {
                        if (!FileList.ContainsKey(namesfilename))
                        {
                            Exception err = new Exception("Error save stats: names file " + namesfilename + " not found for stat file " + statfilename);
                            throw err;
                        }
                        st_namesfile = FileList[namesfilename];
                        if (!st_namesfile.IsNames)
                        {
                            Exception err = new Exception("Error save stats: names file " + namesfilename + " should have flag 'names' into file table ");
                            throw err;
                        }
                        colCode = st.data.Columns.IndexOf("Code");
                        colName = st.data.Columns.IndexOf("Name");
                    }

                    //1. Save header
                    if (st.HaveHeader)
                    {
                        string[] cells = new string[fields.Length];
                        for (int i = 0; i < fields.Length; i++)
                            if ((StatMetadata.ExType)fields[i][colExType] != StatMetadata.ExType.CodeFunction)
                                cells[i] = ((string)fields[i][statfields.data.Columns.IndexOf("Header column name")]);
                        filecontent.AppendLine(String.Join(",", cells));
                    }
                    else
                    {
                        if (st.stored_first_line != "")
                            filecontent.AppendLine(st.stored_first_line);
                    }

                    //2. Save data
                    foreach (DataRow row in st.data.Rows)
                    {
                        //2.1 save stat-files
                        
                        string[] cells = new string[fields.Length];
                        for (int i = 0; i < fields.Length; i++)
                        {
                            if ((StatMetadata.ExType)fields[i][colExType] != StatMetadata.ExType.CodeFunction)
                                cells[i] = (row[fields[i][capt_col_index].ToString()].ToString());//used field by name, beacause not all fields should be saved
                        }
                        filecontent.AppendLine(String.Join(",", cells));

                        //2.2 Save names
                        if (SaveNamesToo) 
                        {
                            string item_code = row[colCode].ToString();
                            string item_Name = row[colName].ToString();
                            bool NamesFileWasChanged = false;
                            if (st_namesfile.names_data.ContainsKey(item_code))
                            {
                                if (st_namesfile.names_data[item_code] != item_Name)
                                {
                                    st_namesfile.names_data[item_code] = item_Name;
                                    NamesFileWasChanged = true;
                                }
                            }
                            else //add new code-name pair into names file
                            {
                                /* fix problem with names when 1 stat file can have names in both names file
                                */
                                //1. find anoter file with names and check - may be current name exists in that file
                                bool FoundInAnotherNamesFile = false;
                                foreach (string another_namesfile in this.FileList.Keys)
                                {
                                    if (FileList[another_namesfile].IsNames 
                                        && another_namesfile != namesfilename && FileList[another_namesfile].Loaded)
                                    {
                                        FoundInAnotherNamesFile = FileList[another_namesfile].names_data.ContainsKey(item_code);
                                        break;
                                    }

                                }
                                //2. Add new name only if it does not exists in possible another names files
                                if (!FoundInAnotherNamesFile)
                                {
                                    st_namesfile.names_data.Add(item_code, item_Name);
                                    NamesFileWasChanged = true;
                                }
                            }
                            if(NamesFileWasChanged)
                                if (!NamesFiles_ToSave.Contains(namesfilename))
                                    NamesFiles_ToSave.Add(namesfilename);
                            if (!st_namesfile.Loaded)
                                st_namesfile.Loaded = true;
                        }
                        
                    }   
                    zip.UpdateEntry(st.path, filecontent.ToString());  
                }
                if (st.IsNames && st.NeedSaveInAnyCase) //save names file even if it was not changed (occurs when names was loaded manually)
                    if (!NamesFiles_ToSave.Contains(statfilename))
                        NamesFiles_ToSave.Add(statfilename);

            }//end of files cycle

            //save names files
            foreach (string names_file in NamesFiles_ToSave)
            {
                if (FileList[names_file].Loaded)
                {
                    string path = FileList[names_file].path;
                    string filecontent = "";
                    string name;
                    foreach (string code in FileList[names_file].names_data.Keys)
                    {
                        name = FileList[names_file].names_data[code].Trim();
                        if (name == "") //WZ crush if name empty
                            name = "Error: name not found";
                        filecontent += code + "\t_(\"" + name + "\")\n";
                    }
                    zip.UpdateEntry(path, filecontent);
                }
            }

            zip.UseZip64WhenSaving = Zip64Option.Never;
            zip.CompressionMethod = CompressionMethod.Deflate; 
            zip.Save(wzfilename);
            zip.Dispose();
        }

        public bool CheckForLoadedFiles()
        {
            foreach (string filename in FileList.Keys)
            {
                if (FileList[filename].Loaded)
                    return true;
            }
            return false;
        }

        private void SetLogging(string statfile)
        {
            if (!FileList.ContainsKey("stateditor_Changelog.txt"))
            {
                Exception err = new Exception("File stateditor_Changelog.txt was not found in file table. You should add this file to filelist.csv");
                throw err;
            }
            StatFile flog = FileList["stateditor_Changelog.txt"];
            if (!flog.Loaded)
            {
                flog.data = new DataTable("Operations log");
                flog.data.Columns.Add("filename", typeof(String));
                flog.data.Columns.Add("filename", typeof(String));
            }
        }

        public bool CheckLinks(out string ErrorMessages)
        {
            ErrorMessages = "";
            return true;
        }

        public DataTable CheckStats()
        {
            DataTable dt_res = new DataTable();
            dt_res.Columns.Add("File", typeof(String));
            dt_res.Columns.Add("Text",typeof(String));
            int capt_col_index = statfields.data.Columns.IndexOf("Caption");
            int colExType = statfields.data.Columns.IndexOf("ExType");
            int colEx = statfields.data.Columns.IndexOf("Expression");
            foreach (string statfile in FileList.Keys)
            {
                DataRow[] fields = statfields.SelectFields(statfile);
                if (FileList[statfile].Loaded && !FileList[statfile].IsNames)
                {

                    //maximum lenght of file
                    if (FileList[statfile].data.Rows.Count > FileList[statfile].Checking_MaxLines)
                    {
                        DataRow row = dt_res.NewRow();
                        row["File"] = statfile;
                        row["Text"] = "This mod will not work in WZ 2.3.9! \nExceeded maximum size " +
                            FileList[statfile].Checking_MaxLines.ToString() + " of stat-file. \nFile description:" + FileList[statfile].FileDescription;
                        dt_res.Rows.Add(row);
                    }

                    //check unique NOT WORK, and NOT SHOW ERROR ROWS
                    //if (FileList[statfile].KeyColumns != null)
                    //    if (FileList[statfile].KeyColumns.Length == 1)
                    //    {
                    //        int key_index = FileList[statfile].KeyColumns[0];
                    //        try
                    //        {
                    //            FileList[statfile].data.Columns[key_index].Unique = true;
                    //        }
                    //        catch (Exception err)
                    //        {
                    //            DataRow row = dt_res.NewRow();
                    //            row["File"] = statfile;
                    //            row["Text"] = "Probably mod can crush WZ! \n Non-unique value inserted in unique column: "
                    //                + FileList[statfile].data.Columns[key_index].ColumnName
                    //                + " of stat-file. \nFile description:" + FileList[statfile].FileDescription;
                    //            dt_res.Rows.Add(row);
                    //        }
                    //    }

                    //check

                    
                    if(IsFileLoaded(statfile))
                    {
                        foreach (DataRow field in fields)
                        {
                            //CheckLink tables 
                            if ((StatMetadata.ExType)field[colExType] == StatMetadata.ExType.LinkTable)
                            {
                                string link_expr = field[colEx].ToString();
                                if(link_expr != "")
                                {
                                    Linker linker = new Linker(link_expr, (string)field[capt_col_index]);
                                    string err = "";

                                    if(!linker.CheckLink(this,statfile,out err))
                                    {
                                        DataRow row = dt_res.NewRow();
                                        row["File"] = statfile;
                                        row["Text"] = "Error in checking link: " + err;
                                        dt_res.Rows.Add(row);
                                    }else
                                    {
                                        DataTable r_dt = FileList[statfile].data;
                                        DataTable dt_link = FileList[linker.link_file].data;
                                        //DataTable dt_link = FileList[lff.lfile].data;
                                        for(int i=0; i<dt_link.Rows.Count; i++)
                                        {
                                            DataRow[] r_rows = r_dt.Select("[" + linker.rkey + "] = '"
                                                                                    + dt_link.Rows[i][linker.link_rkey].ToString() + "'");
                                            if (r_rows.Length == 0   //header line could be stored like first row
                                                &&    String.Join(",", dt_link.Rows[i].ItemArray) != FileList[linker.link_file].stored_first_line
                                                &&  i != 0) //dont check first line (in master first line is header line)
                                            {
                                                
                                                DataRow row = dt_res.NewRow();
                                                row["File"] = statfile;
                                                row["Text"] = String.Format("Error (linked table invalid right key): file={0}, KeyValue={1}, Row={2}",
                                                    linker.link_file, dt_link.Rows[i][linker.link_rkey].ToString(),
                                                    String.Join(",", dt_link.Rows[i].ItemArray));
                                                dt_res.Rows.Add(row);
                                            }

                                        }
                                    }
                                }
                            }

                            //check combo-boxes
                            if ((StatMetadata.ExType)field[colExType] == StatMetadata.ExType.ListFromFile)
                            {
                                ListFromFile lff = new ListFromFile((string)field[colEx]);
                                if (IsFileLoaded(lff.lfile))
                                {
                                    DataTable r_dt = FileList[statfile].data;
                                    DataTable dt_l = FileList[lff.lfile].data;
                                    foreach(DataRow r_row in r_dt.Rows)
                                    {
                                        if (r_row[(string)field[capt_col_index]].ToString().Trim() == "0")
                                            continue; //value 0 means null reference
                                        DataRow[] link_rows = dt_l.Select("[" + lff.lKeyColumn + "] = '"
                                                                            + r_row[(string)field[capt_col_index]].ToString() + "'");
                                        if (link_rows.Length == 0)
                                        {
                                            DataRow row = dt_res.NewRow();
                                            row["File"] = statfile;
                                            row["Text"] = String.Format("Error: key not found in linked table. FieldName={0}, KeyValue={1}, Row={2}",
                                                (string)field[capt_col_index], r_row[(string)field[capt_col_index]].ToString(), String.Join(",",r_row.ItemArray));
                                            dt_res.Rows.Add(row);
                                        }
                                    }
                                
                                }
                            
                            }
                        }
                    }


                    //Check research references

                }

            }

            return dt_res;
        }

        public void CodeFunctions_PerformForcedCalculations()
        {
            foreach (CodeFunction cf in this._codeFunctions)
                cf.PerformForcedUpdate();
        }

    } //end of Stats class
} //end of WZStats namespace
